@@ -1,4 +1,4 @@
-/* $Id: ncbiconf_msvc.h 554121 2017-12-28 16:18:54Z ucko $
+/* $Id: ncbiconf_msvc.h 602172 2020-02-18 15:13:29Z ucko $
  * By Denis Vakatov, NCBI (vakatov@ncbi.nlm.nih.gov)
  *
  * MS-Win 32/64, MSVC++ 6.0/.NET
@@ -69,15 +69,12 @@
 #define HAVE_WSTRING     1
 #define HAVE_SIGNAL_H    1
 
-#define NCBI_DEPRECATED __declspec(deprecated)
-#define NCBI_NORETURN   __declspec(noreturn)
 #define NCBI_FORCEINLINE __forceinline
 #define NCBI_PACKED_ENUM_TYPE(type)  : type
 #define NCBI_PACKED_ENUM_END()
 #define NCBI_RESTRICT_C
 #define NCBI_RESTRICT_CXX
 #define NCBI_TLS_VAR __declspec(thread)
-#define NCBI_WARN_UNUSED_RESULT _Check_return_
 
 #ifdef _WIN64
 #  define HOST         "i386-pc-win64"
