@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMTX__HPP
 #define CORELIB___NCBIMTX__HPP
 
-/* $Id: ncbimtx.hpp 567287 2018-07-17 13:02:49Z gouriano $
+/* $Id: ncbimtx.hpp 602180 2020-02-18 16:35:31Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -239,7 +239,7 @@ public:
     };
 
     /// Translate from the error code value to its string representation.
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
 
     // Standard exception boilerplate code.
     NCBI_EXCEPTION_DEFAULT(CMutexException,CCoreException);
@@ -311,19 +311,19 @@ struct SSystemFastMutex
     // Methods for throwing exceptions, to make inlined methods lighter
 
     /// Throw uninitialized ("eUninitialized") exception.
-    NCBI_XNCBI_EXPORT NCBI_NORETURN
+    NCBI_NORETURN NCBI_XNCBI_EXPORT
     static void ThrowUninitialized(void);
 
     /// Throw lock failed("eLocked") exception.
-    NCBI_XNCBI_EXPORT NCBI_NORETURN
+    NCBI_NORETURN NCBI_XNCBI_EXPORT
     static void ThrowLockFailed(void);
 
     /// Throw unlock failed("eUnlocked") exception.
-    NCBI_XNCBI_EXPORT NCBI_NORETURN
+    NCBI_NORETURN NCBI_XNCBI_EXPORT
     static void ThrowUnlockFailed(void);
 
     /// Throw try lock failed("eTryLock") exception.
-    NCBI_XNCBI_EXPORT NCBI_NORETURN
+    NCBI_NORETURN NCBI_XNCBI_EXPORT
     static void ThrowTryLockFailed(void);
 
 #if !defined(NCBI_OS_MSWIN)
@@ -435,7 +435,7 @@ struct SSystemMutex
     // throw exception eOwner
 
     /// Throw not owned("eOwner") exception.
-    NCBI_XNCBI_EXPORT NCBI_NORETURN
+    NCBI_NORETURN NCBI_XNCBI_EXPORT
     static void ThrowNotOwned(void);
 
 #if !defined(NCBI_OS_MSWIN)
@@ -1540,7 +1540,7 @@ public:
     };
 
     /// Translate from the error code value to its string representation.
-    virtual const char* GetErrCodeString(void) const;
+    virtual const char* GetErrCodeString(void) const override;
 
     // Standard exception boilerplate code.
     NCBI_EXCEPTION_DEFAULT(CConditionVariableException,CCoreException);
