@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBISTRE__HPP
 #define CORELIB___NCBISTRE__HPP
 
-/*  $Id: ncbistre.hpp 562846 2018-04-28 17:39:52Z lavr $
+/*  $Id: ncbistre.hpp 606328 2020-04-20 16:27:53Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -66,14 +66,11 @@
 #  if defined(NCBI_USE_OLD_IOSTREAM)
 #    undef NCBI_USE_OLD_IOSTREAM
 #  endif
-#  if defined(NCBI_COMPILER_GCC)
-#    if NCBI_COMPILER_VERSION < 300
-#      define NO_PUBSYNC
-#    elif NCBI_COMPILER_VERSION >= 310
+#  if defined(NCBI_COMPILER_GCC)  ||  \
+    (defined(NCBI_COMPILER_ANY_CLANG)  &&  defined(__GLIBCXX__))
 // Don't bug us about including <strstream>.
-#      define _CPP_BACKWARD_BACKWARD_WARNING_H 1
-#      define _BACKWARD_BACKWARD_WARNING_H 1
-#    endif
+#    define _CPP_BACKWARD_BACKWARD_WARNING_H 1
+#    define _BACKWARD_BACKWARD_WARNING_H 1
 #  endif
 #  include <iostream>
 #  include <fstream>
@@ -187,12 +184,24 @@ public:
     ) : IO_PREFIX::ifstream(
             ncbi_Utf8ToWstring(_Filename).c_str(), _Mode, _Prot) {
     }
+    explicit CNcbiIfstream(
+        const string& _Filename,
+        IOS_BASE::openmode _Mode = IOS_BASE::in,
+        int _Prot = (int)IOS_BASE::_Openprot
+    ) : CNcbiIfstream(_Filename.c_str(), _Mode, _Prot) {
+    }
     explicit CNcbiIfstream(
         const wchar_t *_Filename,
         IOS_BASE::openmode _Mode = IOS_BASE::in,
         int _Prot = (int)IOS_BASE::_Openprot
     ) : IO_PREFIX::ifstream(_Filename,_Mode,_Prot) {
     }
+    explicit CNcbiIfstream(
+        const wstring& _Filename,
+        IOS_BASE::openmode _Mode = IOS_BASE::in,
+        int _Prot = (int)IOS_BASE::_Openprot
+    ) : CNcbiIfstream(_Filename.c_str(),_Mode,_Prot) {
+    }
  
     void open(
         const char *_Filename,
@@ -201,11 +210,22 @@ public:
         IO_PREFIX::ifstream::open(
             ncbi_Utf8ToWstring(_Filename).c_str(), _Mode, _Prot);
     }
+    void open(
+        const string& _Filename,
+        IOS_BASE::openmode _Mode = IOS_BASE::in,
+        int _Prot = (int)IOS_BASE::_Openprot) {
+        CNcbiIfstream::open(_Filename.c_str(), _Mode, _Prot);
+    }
     void open(const wchar_t *_Filename,
         IOS_BASE::openmode _Mode = IOS_BASE::in,
         int _Prot = (int)ios_base::_Openprot) {
         IO_PREFIX::ifstream::open(_Filename,_Mode,_Prot);
     }
+    void open(const wstring& _Filename,
+        IOS_BASE::openmode _Mode = IOS_BASE::in,
+        int _Prot = (int)ios_base::_Openprot) {
+        CNcbiIfstream::open(_Filename.c_str(), _Mode, _Prot);
+    }
 };
 #elif defined(NCBI_COMPILER_MSVC)
 #  if _MSC_VER >= 1200  &&  _MSC_VER < 1300
@@ -260,12 +280,24 @@ public:
     ) : IO_PREFIX::ofstream(
             ncbi_Utf8ToWstring(_Filename).c_str(), _Mode, _Prot) {
     }
+    explicit CNcbiOfstream(
+        const string& _Filename,
+        IOS_BASE::openmode _Mode = IOS_BASE::out,
+        int _Prot = (int)IOS_BASE::_Openprot
+    ) : CNcbiOfstream(_Filename.c_str(), _Mode, _Prot) {
+    }
     explicit CNcbiOfstream(
         const wchar_t *_Filename,
         IOS_BASE::openmode _Mode = IOS_BASE::out,
         int _Prot = (int)IOS_BASE::_Openprot
     ) : IO_PREFIX::ofstream(_Filename,_Mode,_Prot) {
     }
+    explicit CNcbiOfstream(
+        const wstring& _Filename,
+        IOS_BASE::openmode _Mode = IOS_BASE::out,
+        int _Prot = (int)IOS_BASE::_Openprot
+    ) : CNcbiOfstream(_Filename.c_str(),_Mode,_Prot) {
+    }
  
     void open(
         const char *_Filename,
@@ -274,11 +306,22 @@ public:
         IO_PREFIX::ofstream::open(
             ncbi_Utf8ToWstring(_Filename).c_str(), _Mode, _Prot);
     }
+    void open(
+        const string& _Filename,
+        IOS_BASE::openmode _Mode = IOS_BASE::out,
+        int _Prot = (int)IOS_BASE::_Openprot) {
+        CNcbiOfstream::open(_Filename.c_str(), _Mode, _Prot);
+    }
     void open(const wchar_t *_Filename,
         IOS_BASE::openmode _Mode = IOS_BASE::out,
         int _Prot = (int)IOS_BASE::_Openprot) {
         IO_PREFIX::ofstream::open(_Filename,_Mode,_Prot);
     }
+    void open(const wstring& _Filename,
+        IOS_BASE::openmode _Mode = IOS_BASE::out,
+        int _Prot = (int)IOS_BASE::_Openprot) {
+        CNcbiOfstream::open(_Filename.c_str(), _Mode, _Prot);
+    }
 };
 #elif defined(NCBI_COMPILER_MSVC)
 #  if _MSC_VER >= 1200  &&  _MSC_VER < 1300
@@ -439,6 +482,10 @@ extern CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, char delim,
                                  string::size_type* count = NULL);
 
 /// Read from "is" to "str" up to any symbol contained within "delims" (or EOF)
+/// @note
+///  Special case -- if two different delimiters are back to back and in the
+///  same order as in delims, treat them as a single delimiter. E.g. "\r\n"
+///  will handle mixed DOS/MAC/UNIX line endings such as "\r", "\n" and "\r\n".
 NCBI_XNCBI_EXPORT
 extern CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str,
                                  const string& delims,
@@ -457,7 +504,7 @@ extern CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str,
 /// with all of its _available_ contents (including none) written to "os";
 /// "false" if either extraction from "is" or insertion into "os" failed.
 ///
-/// Note that input ("is") stream state is not always asserted accurately, in
+/// Note that input ("is") stream state is not always asserted accurately:  in
 /// particular, upon successful completion "is.eof()" may not necessarily be
 /// true.  The call may throw exceptions only if they are enabled on the
 /// respective stream(s).
@@ -467,13 +514,13 @@ extern CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str,
 /// which severely lacks error checking (esp. for partial write failures).
 ///
 /// NOTE that the call (as well as the mentioned STL counterpart) provides
-/// only a mechanism of delivering data to the destination "os" stream(buf);
-/// and the successful return result does not generally guarantee that the
-/// data have yet reached the physical destination.  Other "os"-specific API
-/// must be performed to assure the data integrity at the receiving device;
-/// such as checking for errors after doing a "close()" on an ofstream "os".
-/// E.g. data uploading into the Toolkit FTP stream must be finalized with a
-/// read for the byte count delivered;  otherwise, it may not work correctly.
+/// only a mechanism of delivering data to the destination stream(buf) "os";
+/// and a successful return result does not generally guarantee that the data
+/// have yet reached the physical destination.  Other "os"-specific API must be
+/// performed to assure the data integrity at the receiving device:  such as
+/// checking for errors after doing a "close()" on an ofstream "os".  E.g. data
+/// uploading into the Toolkit FTP stream must be finalized with a read for the
+/// byte count delivered;  otherwise, it may not work correctly.
 /// @sa
 ///   CConn_IOStream
 NCBI_XNCBI_EXPORT
@@ -618,18 +665,18 @@ CT_POS_TYPE NcbiInt8ToStreampos(Int8 pos)
 }
 
 
-// CNcbiOstrstreamToString class helps to convert CNcbiOstream buffer to string
-// Sample usage:
+/// CNcbiOstrstreamToString class helps convert CNcbiOstrstream to a string
+/// Sample usage:
 /*
 string GetString(void)
 {
-    CNcbiOstrstream buffer;
-    buffer << "some text";
-    return CNcbiOstrstreamToString(buffer);
+    CNcbiOstrstream out;
+    out << "some text";
+    return CNcbiOstrstreamToString(out);
 }
 */
-// Note: there is no requirement to put '\0' char at the end of buffer;
-//       there is no need to explicitly "unfreeze" the "out" stream.
+/// Note: there is no need to terminate with '\0' char ("ends");
+///       there is no need to explicitly "unfreeze" the "out" stream.
 
 class NCBI_XNCBI_EXPORT CNcbiOstrstreamToString
 {
