@@ -1,7 +1,7 @@
 #ifndef OBJMGR_IMPL_SCOPE_IMPL__HPP
 #define OBJMGR_IMPL_SCOPE_IMPL__HPP
 
-/*  $Id: scope_impl.hpp 576250 2018-12-12 13:23:09Z ivanov $
+/*  $Id: scope_impl.hpp 606922 2020-04-28 18:58:25Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -421,8 +421,6 @@ private:
 
     void x_AttachToOM(CObjectManager& objmgr);
     void x_DetachFromOM(void);
-    void x_RemoveFromHistory(CRef<CTSE_ScopeInfo> tse_info,
-                             int action); // CScope::EActionIfLocked
 
     // clean some cache entries when new data source is added
     void x_ReportNewDataConflict(const CSeq_id_Handle* conflict_id = 0);
@@ -438,9 +436,9 @@ private:
                                const CSeq_entry_Info& new_entry);
 public:
     void x_ClearCacheOnRemoveData(const CTSE_Info* old_tse = 0);
-private:
     void x_ClearAnnotCache(void);
     void x_ClearCacheOnNewAnnot(const CTSE_Info& new_tse);
+private:
     void x_ClearCacheOnRemoveSeqId(const CSeq_id_Handle& id,
                                    CBioseq_ScopeInfo& seq);
     void x_ClearCacheOnRemoveAnnot(const CTSE_Info& old_tse);
@@ -579,14 +577,21 @@ private:
     typedef CBioseq_ScopeInfo::TTSE_MatchSet TTSE_MatchSet;
     typedef CDataSource::TTSE_LockMatchSet TTSE_LockMatchSet_DS;
 
+    CBioseq_ScopeInfo::TAnnotRefInfo&
+        x_GetAnnotRef_Info(const SAnnotSelector* sel,
+                           CBioseq_ScopeInfo::TAnnotRefInfo& main_info,
+                           CBioseq_ScopeInfo::TNAAnnotRefInfo& na_info);
+    
     void x_GetTSESetWithAnnots(TTSE_LockMatchSet& lock,
-                               CBioseq_ScopeInfo& binfo);
+                               CBioseq_ScopeInfo& binfo,
+                               const SAnnotSelector* sel = 0);
     void x_GetTSESetWithAnnots(TTSE_LockMatchSet& lock,
                                TTSE_MatchSet* save_match,
                                CBioseq_ScopeInfo& binfo,
                                const SAnnotSelector* sel = 0);
     void x_GetTSESetWithAnnots(TTSE_LockMatchSet& lock,
-                               TSeq_idMapValue& info);
+                               TSeq_idMapValue& info,
+                               const SAnnotSelector* sel = 0);
     void x_GetTSESetWithAnnots(TTSE_LockMatchSet& lock,
                                TTSE_MatchSet* save_match,
                                TSeq_idMapValue& info,
@@ -634,8 +639,8 @@ private:
 
     IScopeTransaction_Impl* m_Transaction;
 
-    int m_BioseqChangeCounter;
-    int m_AnnotChangeCounter;
+    volatile int m_BioseqChangeCounter;
+    volatile int m_AnnotChangeCounter;
     bool m_KeepExternalAnnotsForEdit;
 
     friend class CScope;
