@@ -92,7 +92,8 @@ public:
         eAdaptive = 0,
         eInternal = 1,
         eExternal = 2,
-        eExhaustive = 3
+        eExhaustive = 3,
+        eIncremental = 4
     };
 
     enum EFlags {
@@ -106,7 +107,8 @@ public:
         fHideMiscFeats =    64,
         fShowSNPFeats =    128,
         fShowCDDFeats =    256,
-        fGeneRNACDSOnly =  512
+        fGeneRNACDSOnly =  512,
+        fHideGapFeats =   1024
     };
     typedef int TFlags; // Binary "OR" of EFlags
 
@@ -167,6 +169,8 @@ public:
 
     const vector<CRef<CSeqsetIndex>>& GetSeqsetIndices(void);
 
+    bool DistributedReferences(void);
+
     // Check all Bioseqs for failure to fetch remote sequence components or feature annotation
     bool IsFetchFailure(void);
 
@@ -221,6 +225,8 @@ public:
     CRef<CBioseqIndex> GetBioseqIndex (const string& accn);
     // Get Bioseq index by handle
     CRef<CBioseqIndex> GetBioseqIndex (CBioseq_Handle bsh);
+    // Get Bioseq index by string
+    CRef<CBioseqIndex> GetBioseqIndex (string& str);
     // Get Bioseq index by feature
     CRef<CBioseqIndex> GetBioseqIndex (const CMappedFeat& mf);
 
@@ -252,6 +258,8 @@ public:
 
     bool IsSmallGenomeSet (void) const { return m_IsSmallGenomeSet; }
 
+    bool DistributedReferences (void) const { return m_DistributedReferences; }
+
     // Check all Bioseqs for failure to fetch remote sequence components or remote feature annotation
     bool IsFetchFailure(void);
 
@@ -264,7 +272,7 @@ private:
     void x_Init (void);
 
     // Recursive exploration to populate vector of index objects for Bioseqs in Seq-entry
-    void x_InitSeqs (const CSeq_entry& sep, CRef<CSeqsetIndex> prnt);
+    void x_InitSeqs (const CSeq_entry& sep, CRef<CSeqsetIndex> prnt, int level = 0);
 
     CRef<CSeq_id> x_MakeUniqueId(void);
 
@@ -303,6 +311,8 @@ private:
     bool m_HasOperon;
     bool m_IsSmallGenomeSet;
 
+    bool m_DistributedReferences;
+
     mutable CAtomicCounter m_Counter;
 
     bool m_IndexFailure;
@@ -403,6 +413,8 @@ public:
     // Feature exploration iterator
     template<typename Fnc> size_t IterateFeatures (Fnc m);
 
+    template<typename Fnc> size_t IterateFeaturesByLoc (const CSeq_loc& slp, Fnc m);
+
     // Getters
     CBioseq_Handle GetBioseqHandle (void) const { return m_Bsh; }
     const CBioseq& GetBioseq (void) const { return m_Bsp; }
@@ -572,6 +584,8 @@ private:
 
     // Common feature collection, delayed until actually needed
     void x_InitFeats (void);
+    // Collect features by location
+    void x_InitFeatsByLoc (const CSeq_loc& slp);
 
     // Set BioSource flags
     void x_InitSource (void);
@@ -878,6 +892,9 @@ public:
     // Map from feature to CFeatureIndex for best gene using CFeatTree in parent CBioseqIndex
     CRef<CFeatureIndex> GetBestGene (void);
 
+    // Map from feature to CFeatureIndex for best VDJC parent using CFeatTree in parent CBioseqIndex
+    CRef<CFeatureIndex> GetBestParent (void);
+
     // Find CFeatureIndex object for overlapping source feature using internal CFeatTree
     CRef<CFeatureIndex> GetOverlappingSource (void);
 
@@ -1054,6 +1071,28 @@ size_t CBioseqIndex::IterateFeatures (Fnc m)
     return count;
 }
 
+// Visit CFeatureIndex objects for range of features
+template<typename Fnc>
+inline
+size_t CBioseqIndex::IterateFeaturesByLoc (const CSeq_loc& slp, Fnc m)
+
+{
+    int count = 0;
+    try {
+        // Delay feature collection until first request, but do not bail on m_FeatsInitialized flag
+        x_InitFeatsByLoc(slp);
+
+        for (auto& sfx : m_SfxList) {
+            count++;
+            m(*sfx);
+        }
+    }
+    catch (CException& e) {
+        LOG_POST(Error << "Error in CBioseqIndex::IterateFeaturesByLoc: " << e.what());
+    }
+    return count;
+}
+
 template<typename Fnc>
 inline
 void CWordPairIndexer::IterateNorm (Fnc m)
