@@ -1,4 +1,4 @@
-/*  $Id: blastdb_dataextract.hpp 564635 2018-05-31 13:13:47Z rackerst $
+/*  $Id: blastdb_dataextract.hpp 591546 2019-08-16 16:59:06Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -75,7 +75,7 @@ public:
         m_Oid2Pig.first = -1;
         m_Gi2SeqIdMap.first = -1;
 
-        CNcbiApplication* app = CNcbiApplication::Instance();
+        CNcbiApplicationAPI* app = CNcbiApplicationAPI::Instance();
         if (app) {
             const CNcbiRegistry& registry = app->GetConfig();
             m_UseLongSeqIds = (registry.Get("BLAST", "LONG_SEQID") == "1");
