@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 
-/*  $Id: seqdb.hpp 579716 2019-02-05 16:53:21Z ivanov $
+/*  $Id: seqdb.hpp 605340 2020-04-09 16:06:43Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -762,6 +762,13 @@ public:
                          ESeqDBAllocType    strategy,
                          TSequenceRanges  * masks = NULL) const;
 
+    int GetAmbigPartialSeq(int                oid,
+                           char            ** buffer,
+                           int                nucl_code,
+                           ESeqDBAllocType    strategy,
+                           TSequenceRanges  * partial_ranges,
+                           TSequenceRanges  * masks = NULL) const;
+
     /// Returns any resources associated with the sequence.
     ///
     /// Calls to GetSequence (but not GetBioseq())
