@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 
-/*  $Id: seqdbcommon.hpp 581731 2019-03-05 16:42:09Z ivanov $
+/*  $Id: seqdbcommon.hpp 605335 2020-04-09 16:04:38Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -84,12 +84,14 @@ public:
         eMemErr,
 
         /// DB version error
-        eVersionErr
+        eVersionErr,
 
+        /// No Tax Id Found
+        eTaxidErr
     };
 
     /// Get a message describing the situation leading to the throw.
-    virtual const char* GetErrCodeString() const
+    virtual const char* GetErrCodeString() const override
     {
         switch ( GetErrCode() ) {
         case eArgErr:  return "eArgErr";
@@ -559,6 +561,13 @@ public:
     	return m_ListInfo;
     }
 
+    void SetSiList( const vector<string> & new_list )
+    {
+        m_SisOids.clear();
+        ITERATE(vector<string>, itr, new_list) {
+        	m_SisOids.push_back(*itr);
+        }
+    }
 protected:
     /// Indicates the current sort order, if any, of this container.
     ESortOrder m_CurrentOrder;
@@ -1023,6 +1032,13 @@ public:
         m_Pigs = new_list;
     }
 
+    void SetSiList( const vector<string> & new_list )
+    {
+        m_Sis.clear();
+        m_Sis.reserve( new_list.size() );
+        m_Sis = new_list;
+    }
+
     /// Build ID set for this negative list.
     const vector<TTi> & GetTiList()
     {
