@@ -1,7 +1,7 @@
 #ifndef CLEANUP___CLEANUP__HPP
 #define CLEANUP___CLEANUP__HPP
 
-/*  $Id: cleanup.hpp 580121 2019-02-11 16:07:50Z ivanov $
+/*  $Id: cleanup.hpp 605251 2020-04-08 14:24:56Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -105,6 +105,9 @@ public:
     CConstRef<CCleanupChange> BasicCleanup(CBioSource& src,   Uint4 options = 0);
 	// Cleanup a Submit-block
 	CConstRef<CCleanupChange> BasicCleanup(CSubmit_block& block, Uint4 options = 0);
+    // Cleanup descriptors
+    CConstRef<CCleanupChange> BasicCleanup(CSeqdesc& desc, Uint4 options = 0);
+    CConstRef<CCleanupChange> BasicCleanup(CSeq_descr & desc, Uint4 options = 0);
 
     // Handle versions.
     CConstRef<CCleanupChange> BasicCleanup(CSeq_entry_Handle& seh, Uint4 options = 0);
@@ -122,7 +125,7 @@ public:
     CConstRef<CCleanupChange> ExtendedCleanup(CSeq_annot& sa,  Uint4 options = 0);
 
     // Handle versions
-    CConstRef<CCleanupChange> ExtendedCleanup(CSeq_entry_Handle& seh, Uint4 options = 0);
+    static CConstRef<CCleanupChange> ExtendedCleanup(CSeq_entry_Handle& seh, Uint4 options = 0);
 
     // Useful cleanup functions
 
@@ -369,7 +372,8 @@ public:
 /// Performs WGS specific cleanup
 /// @param entry Seq-entry to edit
 /// @return Boolean return value indicates whether object was updated
-    static bool WGSCleanup(CSeq_entry_Handle entry, bool instantiate_missing_proteins = true, Uint4 options = 0);
+    static bool WGSCleanup(CSeq_entry_Handle entry, bool instantiate_missing_proteins = true, Uint4 options = 0,
+        bool run_extended_cleanup = true);
 
 /// For table2asn -c s
 /// Adds an exception of "low-quality sequence region" to coding regions
