@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___GATHER_ITEMS__HPP
 #define OBJTOOLS_FORMAT___GATHER_ITEMS__HPP
 
-/*  $Id: gather_items.hpp 568182 2018-07-31 23:53:48Z kans $
+/*  $Id: gather_items.hpp 607397 2020-05-04 14:17:25Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -62,9 +62,11 @@ public:
     // virtual constructor
     static CFlatGatherer* New(CFlatFileConfig::TFormat format);
 
-    virtual void Gather(CFlatFileContext& ctx, CFlatItemOStream& os) const;
     virtual void Gather(CFlatFileContext& ctx, CFlatItemOStream& os,
-                        const CSeq_entry_Handle& entry, bool useSeqEntryIndexing) const;
+                        bool doNuc = true, bool doProt = true) const;
+    virtual void Gather(CFlatFileContext& ctx, CFlatItemOStream& os,
+                        const CSeq_entry_Handle& entry, bool useSeqEntryIndexing,
+                        bool doNuc = true, bool doProt = true) const;
 
     virtual void SetCanceledCallback(const ICanceled* pCanceledCallback) {
         m_pCanceledCallback = pCanceledCallback;
@@ -81,11 +83,15 @@ protected:
     CBioseqContext& Context      (void) const { return *m_Current; }
     const CFlatFileConfig& Config(void) const { return m_Context->GetConfig(); }
 
+    vector<string>& RefCache (void) const { return m_RefCache; }
+
     virtual void x_GatherSeqEntry(CFlatFileContext& ctx,
-        CRef<CTopLevelSeqEntryContext> topLevelSeqEntryContext = CRef<CTopLevelSeqEntryContext>() ) const;
+        CRef<CTopLevelSeqEntryContext> topLevelSeqEntryContext = CRef<CTopLevelSeqEntryContext>(),
+        bool doNuc = true, bool doProt = true ) const;
     virtual void x_GatherSeqEntry(CFlatFileContext& ctx,
         const CSeq_entry_Handle& entry, bool useSeqEntryIndexing,
-        CRef<CTopLevelSeqEntryContext> topLevelSeqEntryContext = CRef<CTopLevelSeqEntryContext>() ) const;
+        CRef<CTopLevelSeqEntryContext> topLevelSeqEntryContext = CRef<CTopLevelSeqEntryContext>(),
+        bool doNuc = true, bool doProt = true ) const;
 
     virtual void x_GatherBioseq(
         const CBioseq_Handle& prev_seq, const CBioseq_Handle& this_seq, const CBioseq_Handle& next_seq, 
@@ -100,9 +106,10 @@ protected:
         const CSeq_loc* loc,
         CRef<feature::CFeatTree> ftree,
         CFeatureItem::EMapped mapped = CFeatureItem::eMapped_not_mapped,
+        bool suppressAccession = false,
         CConstRef<CFeatureItem> parentFeatureItem = CConstRef<CFeatureItem>() ) const
     {
-        return new CFeatureItem( feat, ctx, ftree, loc, mapped, parentFeatureItem );
+        return new CFeatureItem( feat, ctx, ftree, loc, mapped, suppressAccession, parentFeatureItem );
     };
 
     // source/organism
@@ -112,6 +119,7 @@ protected:
     typedef CBioseqContext::TReferences TReferences;
     void x_GatherReferences(void) const;
     void x_GatherReferences(const CSeq_loc& loc, TReferences& refs) const;
+    void x_GatherReferencesIdx(const CSeq_loc& loc, TReferences& refs) const;
     void x_GatherCDSReferences(TReferences& refs) const;
 
     // features
@@ -134,6 +142,8 @@ protected:
         CBioseqContext& ctx) const;
     void x_GatherFeaturesOnRangeIdx(const CSeq_loc& loc, SAnnotSelector& sel,
         CBioseqContext& ctx) const;
+    size_t x_GatherFeaturesOnSegmentIdx(const CSeq_loc& loc, SAnnotSelector& sel,
+        CBioseqContext& ctx) const;
     void x_GatherFeaturesOnRange(const CSeq_loc& loc, SAnnotSelector& sel,
         CBioseqContext& ctx) const;
 
@@ -206,6 +216,8 @@ protected:
     mutable CRef<CBioseqContext>     m_Current;
     mutable TCommentVec              m_Comments;
 
+    mutable vector<string>           m_RefCache;
+
     mutable  CConstRef<CUser_object> m_FirstGenAnnotSCAD;
 
     // Raw pointer because we do NOT own it
