@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___FLAT_SEQLOC__HPP
 #define OBJTOOLS_FORMAT_ITEMS___FLAT_SEQLOC__HPP
 
-/*  $Id: flat_seqloc.hpp 560984 2018-03-29 19:53:54Z dondosha $
+/*  $Id: flat_seqloc.hpp 604098 2020-03-23 12:19:42Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -83,7 +83,7 @@ public:
     // required even when location consists of a single interval.
     CFlatSeqLoc(const CSeq_loc& loc, CBioseqContext& ctx, 
                 TType type = eType_location, bool show_all_accns = false,
-                bool add_join = false);
+                bool add_join = false, bool suppress_accession = false);
 
     const string&     GetString(void)    const { return m_String;    }
     
@@ -106,19 +106,20 @@ private:
 
     bool x_Add(const CSeq_loc& loc, CNcbiOstrstream& oss,
         CBioseqContext& ctx, TType type, bool show_comp,
-        bool show_all_accns = false, bool join_whole_loc = false);
+        bool show_all_accns = false, bool join_whole_loc = false,
+        bool suppress_accession = false);
     bool x_Add(const CSeq_interval& si, CNcbiOstrstream& oss,
         CBioseqContext& ctx, TType type, bool show_comp,
-        bool show_all_accns = false);
+        bool show_all_accns = false, bool suppress_accession = false);
     bool x_Add(const CSeq_point& pnt, CNcbiOstrstream& oss,
         CBioseqContext& ctx, TType type, bool show_comp,
-        bool show_all_accns = false);
+        bool show_all_accns = false, bool suppress_accession = false);
     bool x_Add(TSeqPos pnt, const CInt_fuzz* fuzz, CNcbiOstrstream& oss,
         EHTML html, EForce force = eForce_None, ESource source = eSource_Other,
-        bool show_all_accns = false);
+        bool show_all_accns = false, bool suppress_accession = false);
     void x_AddID(const CSeq_id& id, CNcbiOstrstream& oss,
         CBioseqContext& ctx, TType type,
-        bool show_all_accns = false);
+        bool show_all_accns = false, bool suppress_accession = false);
 
     bool x_IsAccessionVersion( CSeq_id_Handle id );
 
