@@ -1,4 +1,4 @@
- /*  $Id: gff3_reader.hpp 575510 2018-11-29 19:38:03Z ivanov $
+ /*  $Id: gff3_reader.hpp 607807 2020-05-07 18:58:43Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -26,7 +26,7 @@
  * Author: Frank Ludwig
  *
  * File Description:
- *   BED file reader
+ *   GFF3 file reader
  *
  */
 
@@ -34,21 +34,13 @@
 #define OBJTOOLS_READERS___GFF3_READER__HPP
 
 #include <corelib/ncbistd.hpp>
-#include <objects/seq/Annotdesc.hpp>
-#include <objects/seqfeat/Seq_feat.hpp>
-#include <objects/seqfeat/Cdregion.hpp>
-
-#include <objtools/readers/reader_base.hpp>
 #include <objtools/readers/gff2_reader.hpp>
-#include <objtools/readers/gff2_data.hpp>
 
 BEGIN_NCBI_SCOPE
-
 BEGIN_SCOPE(objects) // namespace ncbi::objects::
 
 class CGFFReader;
 class CGff3ReadRecord;
-class SRecord;
 
 //  ============================================================================
 class CGff3ReadRecord
@@ -67,9 +59,22 @@ protected:
         const string& );
 };
 
-//  ----------------------------------------------------------------------------
+//  ============================================================================
+struct SAlignmentData
+//  ============================================================================
+{
+    using MAP_ID_TO_ALIGN = map<string, list<CRef<CSeq_align>>>;
+    MAP_ID_TO_ALIGN mAlignments;
+    using LIST_IDS = list<string>;
+    LIST_IDS mIds;
+
+    void Reset() { mAlignments.clear(); mIds.clear(); };
+    operator bool() const { return !mAlignments.empty(); };
+};
+
+//  ============================================================================
 class NCBI_XOBJREAD_EXPORT CGff3Reader
-//  ----------------------------------------------------------------------------
+//  ============================================================================
     : public CGff2Reader
 {
 public:
@@ -87,13 +92,31 @@ public:
         unsigned int uFlags,
         const string& name = "",
         const string& title = "",
-        SeqIdResolver resolver = CReadUtil::AsSeqId);
+        SeqIdResolver resolver = CReadUtil::AsSeqId,
+        CReaderListener* = nullptr);
+
+    CGff3Reader(
+        unsigned int uFlags,
+        CReaderListener*);
 
     virtual ~CGff3Reader();
 
+    virtual CRef<CSeq_annot>
+    ReadSeqAnnot(
+        ILineReader& lr,
+        ILineErrorListener* pErrors=nullptr);
+
     bool IsInGenbankMode() const;
 
 protected:
+    virtual void xGetData(
+        ILineReader&,
+        TReaderData&);
+
+    virtual void xProcessData(
+        const TReaderData&,
+        CSeq_annot&);
+
     virtual CGff2Record* x_CreateRecord() { return new CGff3ReadRecord(); };    
 
     virtual bool xInitializeFeature(
@@ -102,48 +125,54 @@ protected:
 
     virtual bool xUpdateAnnotFeature(
         const CGff2Record&,
-        CRef< CSeq_annot >,
+        CSeq_annot&,
         ILineErrorListener*);
 
     virtual bool xAddFeatureToAnnot(
         CRef< CSeq_feat >,
-        CRef< CSeq_annot > );
+        CSeq_annot& );
 
     virtual bool xUpdateAnnotExon(
         const CGff2Record&,
         CRef<CSeq_feat>,
-        CRef<CSeq_annot>,
+        CSeq_annot&,
         ILineErrorListener*);
 
     virtual bool xUpdateAnnotCds(
         const CGff2Record&,
         CRef<CSeq_feat>,
-        CRef<CSeq_annot>,
+        CSeq_annot&,
         ILineErrorListener*);
 
     virtual bool xUpdateAnnotGene(
         const CGff2Record&,
         CRef<CSeq_feat>,
-        CRef<CSeq_annot>,
+        CSeq_annot&,
+        ILineErrorListener*);
+
+    virtual bool xUpdateAnnotRegion(
+        const CGff2Record&,
+        CRef<CSeq_feat>,
+        CSeq_annot&,
         ILineErrorListener*);
 
     virtual bool xUpdateAnnotGeneric(
         const CGff2Record&,
         CRef<CSeq_feat>,
-        CRef<CSeq_annot>,
+        CSeq_annot&,
         ILineErrorListener*);
 
     virtual bool xUpdateAnnotMrna(
         const CGff2Record&,
         CRef<CSeq_feat>,
-        CRef<CSeq_annot>,
+        CSeq_annot&,
         ILineErrorListener*);
 
     virtual bool xFindFeatureUnderConstruction(
         const CGff2Record&,
         CRef<CSeq_feat>&);
 
-    virtual bool xVerifyCdsParents(
+    void xVerifyCdsParents(
         const CGff2Record&);
 
     virtual bool xFeatureSetXrefGrandParent(
@@ -158,10 +187,9 @@ protected:
 
     string xNextGenericId();
 
-    bool xVerifyExonLocation(
+    void xVerifyExonLocation(
         const string&,
-        const CGff2Record&,
-        ILineErrorListener*);
+        const CGff2Record&);
 
     virtual bool xIsIgnoredFeatureType(
         const string&);
@@ -169,18 +197,33 @@ protected:
     virtual void xAddPendingExon(
         const string&,
         const CGff2Record&);
+
     virtual void xGetPendingExons(
         const string&,
         list<CGff2Record>&);
-    virtual void xPostProcessAnnot(
-        CRef<CSeq_annot>&,
-        ILineErrorListener*);
 
+    virtual void xPostProcessAnnot(
+        CSeq_annot&);
+
+    void xProcessAlignmentData(
+        CSeq_annot& pAnnot);
+
+    virtual bool xParseFeature(
+        const string&,
+        CSeq_annot&,
+        ILineErrorListener*);
+      
+    virtual bool xParseAlignment(
+        const string& strLine);
+
+    bool xNeedsNewSeqAnnot(
+        const string&);
 
     // Data:
     map<string, string> mCdsParentMap;
     map<string, CRef<CSeq_interval> > mMrnaLocs;
     map<string, string> mIdToSeqIdMap;
+    SAlignmentData mAlignmentData;
 
     using PENDING_EXONS = map<string, list<CGff2Record> >;
     PENDING_EXONS mPendingExons;
