@@ -1,7 +1,7 @@
 #ifndef SERIAL___RPCBASE_IMPL__HPP
 #define SERIAL___RPCBASE_IMPL__HPP
 
-/*  $Id: rpcbase_impl.hpp 578926 2019-01-28 18:17:50Z ivanov $
+/*  $Id: rpcbase_impl.hpp 588592 2019-06-26 18:52:32Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -101,6 +101,8 @@ protected:
     virtual void x_ReadReply(CObjectIStream& in, CSerialObject& reply) = 0;
     virtual string x_GetAffinity(const CSerialObject& request) const = 0;
 
+    static const char* GetContentTypeHeader(ESerialDataFormat format);
+
 private:
     /// Prohibit default copy constructor and assignment operator.
     CRPCClient_Base(const CRPCClient_Base&);
@@ -158,7 +160,7 @@ public:
     NCBI_EXCEPTION_DEFAULT(CRPCClientException, CException);
 
 protected:
-    virtual void x_Assign(const CException& src)
+    virtual void x_Assign(const CException& src) override
     {
         CException::x_Assign(src);
         m_RetryCtx = dynamic_cast<const CRPCClientException&>(src).m_RetryCtx;
