@@ -19,9 +19,12 @@ For more information please visit:  http://bitmagic.io
 */
 
 /*! \file bmconst.h
-    \brief Various constants and tables
+    \brief Constants, tables and typedefs
+    @internal
 */
 
+#include <cstdint>
+
 namespace bm
 {
 
@@ -39,8 +42,12 @@ typedef unsigned short short_t;
 # define BM_DEFAULT_POOL_SIZE 4096
 #endif
 
-
-const unsigned id_max = 0xFFFFFFFF;
+#ifdef BM64ADDR
+const unsigned long long id_max32 = 0xFFFFFFFFull;
+const unsigned long long id_max48 = 0xFFFFFFFFFFFFull;
+#else
+const unsigned id_max32 = 0xFFFFFFFFu;
+#endif
 
 // Data Block parameters
 
@@ -49,9 +56,16 @@ const unsigned set_block_shift = 16u;
 const unsigned set_block_mask  = 0xFFFFu;
 const unsigned set_blkblk_mask = 0xFFFFFFu;
 
-const unsigned set_block_plain_size = set_block_size / 32u;
+// set block size in bytes
+const unsigned set_block_alloc_size = bm::set_block_size * unsigned(sizeof(bm::word_t));
+
+const unsigned set_block_plain_size = bm::set_block_size / 32u;
 const unsigned set_block_plain_cnt = (unsigned)(sizeof(bm::word_t) * 8u);
 
+const unsigned block_waves = 64;
+const unsigned set_block_digest_wave_size = bm::set_block_size / bm::block_waves;
+const unsigned set_block_digest_pos_shift = 10;
+
 // Word parameters
 
 const unsigned set_word_shift = 5u;
@@ -65,34 +79,58 @@ typedef unsigned short gap_word_t;
 const unsigned gap_max_buff_len = 1280;
 const unsigned gap_max_bits = 65536;
 const unsigned gap_equiv_len = (unsigned)
-   ((sizeof(bm::word_t) * bm::set_block_size) / sizeof(gap_word_t));
+   ((sizeof(bm::word_t) * bm::set_block_size) / sizeof(bm::gap_word_t));
+const unsigned gap_max_bits_cmrz = bm::gap_max_bits / 2;
 const unsigned gap_levels = 4;
 const unsigned gap_max_level = bm::gap_levels - 1;
 
+const unsigned bie_cut_off = 16384; // binary interpolative encode size cut-off
+
 
 // Block Array parameters
 
-const unsigned set_array_size = 256u;
+
+const unsigned set_array_size32 = 256u;
+const unsigned set_sub_array_size = set_array_size32;
 const unsigned set_array_shift = 8u;
 const unsigned set_array_mask  = 0xFFu;
-const unsigned set_total_blocks = (bm::set_array_size * bm::set_array_size);
 
-const unsigned bits_in_block = bm::set_block_size * (unsigned)(sizeof(bm::word_t) * 8);
-const unsigned bits_in_array = bm::bits_in_block * bm::set_array_size;
+const unsigned set_total_blocks32 = (bm::set_array_size32 * bm::set_array_size32);
+
+#ifdef BM64ADDR
+const unsigned set_total_blocks48 = bm::id_max48 / bm::gap_max_bits;
+const unsigned long long id_max = bm::id_max48;
+const unsigned long long set_array_size48 = 1 + (bm::id_max48 / (bm::set_sub_array_size * bm::gap_max_bits));
+const unsigned  set_top_array_size = bm::set_array_size48;
+const id64_t set_total_blocks = id64_t(bm::set_top_array_size) * set_sub_array_size;
+//bm::id_max / (bm::gap_max_bits * bm::set_sub_array_size);
+#else
+const unsigned id_max = bm::id_max32;
+const unsigned set_top_array_size = bm::set_array_size32;
+const unsigned set_total_blocks = bm::set_total_blocks32;
+#endif
+
+const unsigned bits_in_block = bm::set_block_size * unsigned((sizeof(bm::word_t) * 8));
+const unsigned bits_in_array = bm::bits_in_block * bm::set_array_size32;
+
+
+// Rank-Select parameters
+const unsigned rs3_border0 = 21824; // 682 words by 32-bits
+const unsigned rs3_border1 = (rs3_border0 * 2); // 43648
+const unsigned rs3_half_span = rs3_border0 / 2;
+
+// misc parameters for sparse vec algorithms
+const unsigned sub_block3_size = bm::gap_max_bits / 4;
 
 
 #if defined(BM64OPT) || defined(BM64_SSE4)
-
 typedef id64_t  wordop_t;
-const id64_t    all_bits_mask = 0xffffffffffffffff;
+const id64_t    all_bits_mask = 0xffffffffffffffffULL;
 const unsigned set_block_size_op  = bm::set_block_size / 2;
-
 #else
-
 typedef word_t wordop_t;
 const word_t all_bits_mask = 0xffffffff;
 const unsigned set_block_size_op  = bm::set_block_size;
-
 #endif
 
 
@@ -106,6 +144,7 @@ enum strategy
     BM_GAP = 1  //!< GAP compression is ON.
 };
 
+
 /**
     Codes of set operations
     @ingroup bvector
@@ -129,6 +168,18 @@ enum set_operation
     set_END
 };
 
+/**
+    Bit operations
+    @ingroup bvector
+*/
+enum operation
+{
+    BM_AND = set_AND,
+    BM_OR  = set_OR,
+    BM_SUB = set_SUB,
+    BM_XOR = set_XOR
+};
+
 
 /*!
    @brief Sort order declaration
@@ -136,10 +187,10 @@ enum set_operation
 */
 enum sort_order
 {
-    BM_UNSORTED = 0,      //!< input set is NOT sorted
-    BM_SORTED = 1,        //!< input set is sorted (ascending order)
-    BM_SORTED_UNIFORM = 2,//!< input set is sorted and belongs to one address block
-    BM_UNKNOWN = 3        //!< sort order unknown
+    BM_UNSORTED       = 0,  //!< input set is NOT sorted
+    BM_SORTED         = 1,  //!< input set is sorted (ascending order)
+    BM_SORTED_UNIFORM = 2,  //!< sorted and in one block (internal!)
+    BM_UNKNOWN        = 3   //!< sort order unknown
 };
 
 
@@ -177,19 +228,20 @@ template<bool T> struct _copyright
 };
 
 template<bool T> const char _copyright<T>::_p[] = 
-    "BitMagic C++ Library. v.3.12.5 (c) 2002-2018 Anatoliy Kuznetsov.";
-template<bool T> const unsigned _copyright<T>::_v[3] = {3, 12, 5};
+    "BitMagic C++ Library. v.6.0.0 (c) 2002-2020 Anatoliy Kuznetsov.";
+template<bool T> const unsigned _copyright<T>::_v[3] = {6, 0, 0};
 
 
-template<bool T> struct DeBruijn_bit_position
-{
-    static const unsigned _multiply[32];
-};
 
 /**
     DeBruijn majic table
     @internal
 */
+template<bool T> struct DeBruijn_bit_position
+{
+    static const unsigned _multiply[32];
+};
+
 template<bool T>
 const unsigned DeBruijn_bit_position<T>::_multiply[32] = { 
   0, 1, 28, 2, 29, 14, 24, 3, 30, 22, 20, 15, 25, 17, 4, 8, 
@@ -251,6 +303,39 @@ const unsigned char bit_count_table<T>::_count[256] = {
 
 //---------------------------------------------------------------------
 
+/** Structure for LZCNT constants (4-bit)
+    @ingroup bitfunc
+*/
+template<bool T> struct lzcnt_table
+{
+    static unsigned char const _lut[16];
+};
+
+template<bool T>
+const unsigned char lzcnt_table<T>::_lut[16] = 
+{
+    32U, 31U, 30U, 30U, 29U, 29U, 29U, 29U,
+    28U, 28U, 28U, 28U, 28U, 28U, 28U, 28U
+};
+
+/** Structure for TZCNT constants 
+    @ingroup bitfunc
+*/
+template<bool T> struct tzcnt_table
+{
+    static unsigned char const _lut[37];
+};
+
+template<bool T>
+const unsigned char tzcnt_table<T>::_lut[37] =
+{ 
+    32, 0, 1, 26, 2, 23, 27, 0, 3, 16, 24, 30, 28, 11,
+    0, 13, 4, 7, 17, 0, 25, 22, 31, 15, 29, 10, 12, 6, 0, 
+    21, 14, 9, 5, 20, 8, 19, 18 
+};
+
+
+
 /** Structure keeps all-left/right ON bits masks. 
     @ingroup bitfunc 
 */
@@ -331,10 +416,55 @@ enum simd_codes
     simd_none  = 0,   ///!< No SIMD or any other optimization
     simd_sse2  = 1,   ///!< Intel SSE2
     simd_sse42 = 2,   ///!< Intel SSE4.2
-    simd_avx2  = 5    ///!< Intel AVX2
+    simd_avx2  = 5,   ///!< Intel AVX2
+    simd_avx512  = 6  ///!< Intel AVX512
 };
 
 
+/*!
+   \brief Byte orders recognized by the library.
+   @internal
+*/
+enum ByteOrder
+{
+    BigEndian    = 0,
+    LittleEndian = 1
+};
+
+/**
+    Internal structure. Different global settings.
+    @internal
+*/
+template<bool T> struct globals
+{
+    struct bo
+    {
+        ByteOrder  _byte_order;
+
+        bo()
+        {
+            unsigned x;
+            unsigned char *s = (unsigned char *)&x;
+            s[0] = 1; s[1] = 2; s[2] = 3; s[3] = 4;
+            if(x == 0x04030201)
+            {
+                _byte_order = LittleEndian;
+                return;
+            }
+            if(x == 0x01020304)
+            {
+                _byte_order = BigEndian;
+                return;
+            }
+            _byte_order = LittleEndian;
+        }
+    };
+
+    static bo  _bo;
+    static ByteOrder byte_order() { return _bo._byte_order; }
+};
+template<bool T> typename globals<T>::bo globals<T>::_bo;
+
 
 } // namespace
 
