@@ -1,7 +1,7 @@
 #ifndef UTIL___RANGE_COLL__HPP
 #define UTIL___RANGE_COLL__HPP
 
-/*  $Id: range_coll.hpp 560849 2018-03-28 12:46:32Z ivanov $
+/*  $Id: range_coll.hpp 601340 2020-02-05 20:26:06Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,9 +44,10 @@ BEGIN_NCBI_SCOPE
 template<class Range, class Position>
 struct PRangeLessPos
 {
-    bool    operator()(const Range &R, Position Pos)     { return R.GetToOpen() <= Pos;  }    
-    bool    operator()(Position Pos, const Range &R)     { return Pos < R.GetToOpen();  }    
-    bool    operator()(const Range &R1, const Range &R2) { return R1.GetToOpen() < R2.GetToOpen();  }    
+    using is_transparent = void;
+    bool operator()(const Range &R, Position Pos)     { return R.GetToOpen() <= Pos;  }
+    //bool operator()(Position Pos, const Range &R)     { return Pos < R.GetToOpen();  }
+    bool operator()(const Range &R1, const Range &R2) { return R1.GetToOpen() < R2.GetToOpen();  }
 };
     
 ///////////////////////////////////////////////////////////////////////////////
