@@ -1,4 +1,4 @@
-/*  $Id: redoalignment_unit_test.cpp 498724 2016-04-19 13:36:57Z ivanov $ 
+/*  $Id: redoalignment_unit_test.cpp 504861 2016-06-20 15:45:40Z boratyng $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -290,7 +290,7 @@ void CRedoAlignmentTestFixture::
 		hitsaving_opts, ext_options->compositionBasedStats,
         scoring_opts->gapped_calculation));
 
-    BlastHSPWriter* writer = BlastHSPWriterNew(&writer_info, NULL);
+    BlastHSPWriter* writer = BlastHSPWriterNew(&writer_info, NULL, NULL);
     BOOST_REQUIRE(writer_info == NULL);
 
     BlastHSPStream* hsp_stream = BlastHSPStreamNew(
