@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: get_species_taxids.sh 581766 2019-03-05 17:46:45Z ivanov $
+# $Id: get_species_taxids.sh 588462 2019-06-24 18:46:42Z camacho $
 # ===========================================================================
 #
 #                            PUBLIC DOMAIN NOTICE
@@ -34,25 +34,25 @@
 #
 # ===========================================================================
 
-export PATH=/bin:/usr/bin:/am/ncbiapdata/bin:$HOME/edirect
+export PATH=/bin:/usr/bin:/am/ncbiapdata/bin:$HOME/edirect:$PATH
 set -uo pipefail
 
 TOO_MANY_MATCHES=500
-OUTPUT=`mktemp -t $(basename -s .sh $0)-XXXXXXX`
-TMP=`mktemp -t $(basename -s .sh $0)-XXXXXXX`
+OUTPUT=`mktemp`
+TMP=`mktemp`
 trap " /bin/rm -fr $OUTPUT $TMP" INT QUIT EXIT HUP KILL ALRM
 
 usage() { 
     echo "$0 usage:"; 
     echo -e "\t-t <taxonomy ID>\n\t\tGet taxonomy IDs at or below input taxonomy ID level"; 
     echo -e "\t-n <Scientific Name, Common Name or Keyword>\n\t\tGet taxonomy information for organism";
-    exit 1; 
+    exit 0; 
 }
 
 error_exit() {
     msg=$1
     exit_code=${2:-1}
-    echo $msg;
+    >&2 echo $msg;
     exit $exit_code;
 }
 
