@@ -1,4 +1,4 @@
-/*  $Id: blastdb_convert.cpp 579231 2019-01-31 16:33:40Z ivanov $
+/*  $Id: blastdb_convert.cpp 598221 2019-12-05 15:33:01Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -282,6 +282,15 @@ void s_GetProfileDBsExt(vector<string> & extns)
 	extns.push_back("wcounts");
 	extns.push_back("obsr");
 }
+void s_CheckDuplicateVols(const vector<string> & vols, const string & vol_name)
+{
+	for(unsigned int i =0; i < vols.size(); i++){
+		if(vols[i] == vol_name){
+			NCBI_THROW(CInvalidDataException, eInvalidInput,
+			            "Duplicate db volumes");
+		}
+	}
+}
 
 int CBlastdbConvertApp::Run(void)
 {
@@ -300,6 +309,7 @@ int CBlastdbConvertApp::Run(void)
     SetDiagPostPrefix("blastdb_convert");
 
     int status = 0;
+    bool cleanup = false;
     try {
 
     	if ( !CMemoryFileMap::IsSupported()) {
@@ -313,12 +323,17 @@ int CBlastdbConvertApp::Run(void)
     		            "changing the output name, please provide a (different) database name ");
     	}
 
-    	if (DeleteBlastDb(kOutputAbsPath, seqtype)) {
-    	        	*m_LogFile << "Deleted existing BLAST database with identical name." << endl;
-        }
     	CBuildDatabase::CreateDirectories(kOutput);
     	m_LogFile = & (args["logfile"].HasValue() ? args["logfile"].AsOutputFile() : cout);
 
+        // Check the appropriate version
+        {{
+            CRef<CSeqDB> source_blastdb(new CSeqDB(kInputDb, seqtype));
+            if (source_blastdb && source_blastdb->GetBlastDbVersion() != eBDB_Version4) {
+                NCBI_THROW(CBlastException, eNotSupported, "Only BLASTDBv4 can be used as input");
+            }
+        }}
+
         vector<string> paths, alias_files;
         bool use_index_in_filename = true;
         // FIXME: add error checking for non-handled cases
@@ -336,6 +351,12 @@ int CBlastdbConvertApp::Run(void)
         if(alias_files.size() == 1) {
         	s_LookForOidlistInAliasFile (alias_files[0]);
         }
+
+    	if (DeleteBlastDb(kOutputAbsPath, seqtype)) {
+    		cerr << kOutputAbsPath << endl;
+    	        	*m_LogFile << "Deleted existing BLAST database with identical name." << endl;
+        }
+
         if(alias_files.size() == 0) {
         	use_index_in_filename = false;
         }
@@ -362,6 +383,7 @@ int CBlastdbConvertApp::Run(void)
         unique_ptr<CWriteDB_TaxID> taxdb (new CWriteDB_TaxID(
         		                              GetFileNameFromExistingLMDBFile(lmdb_fname_w_path, ELMDBFileType::eTaxId2Offsets),
         		                              kMemoryMapSize));
+        cleanup = true;
         for (unsigned int p=0; p < paths.size(); p++) {
         	string & vol_path = paths[p];
             _TRACE("Processing " << vol_path);
@@ -401,8 +423,10 @@ int CBlastdbConvertApp::Run(void)
             	  vol->GetAllTaxIDs(oid, tax_ids);
                   taxdb->InsertEntries(tax_ids, oid_total+oid);
              }
-            vol_names.push_back(kOutputVol);
-            vol_num_oids.push_back(kNumOids);
+
+             s_CheckDuplicateVols(vol_names, kOutputVol);
+             vol_names.push_back(kOutputVol);
+             vol_num_oids.push_back(kNumOids);
              oid_total += kNumOids;
 
              for (auto ext : extns) {
@@ -441,7 +465,7 @@ int CBlastdbConvertApp::Run(void)
 
     } CATCH_ALL(status)
 
-    if(status != 0) {
+    if((status != 0) && cleanup) {
     	DeleteBlastDb(kOutputAbsPath, seqtype);
     }
     return status;
