@@ -1,4 +1,4 @@
-/*  $Id: makeprofiledb.cpp 563704 2018-05-14 15:46:55Z fongah2 $
+/*  $Id: makeprofiledb.cpp 596198 2019-11-04 15:01:48Z boratyng $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -77,6 +77,7 @@ static const string kOutIndexFile("index");
 static const string kObsrThreshold("obsr_threshold");
 static const string kExcludeInvalid("exclude_invalid");
 static const string kBinaryScoremat("binary");
+static const string kUseCmdlineThreshold("force");
 
 static const string kLogFile("logfile");
 
@@ -261,6 +262,7 @@ private:
 
     struct RPS_DbInfo	m_RpsDbInfo;
     CRef<CWriteDB>	m_OutputDb;
+    EBlastDbVersion m_DbVer;
     CRef<CTaxIdSet> m_Taxids;
     bool m_Done;
 
@@ -273,6 +275,7 @@ private:
 	Int4 m_CurrObsrOffset;
 
 	bool m_UpdateFreqRatios;
+	bool m_UseModelThreshold;
 };
 
 CMakeProfileDBApp::CMakeProfileDBApp(void)
@@ -280,9 +283,10 @@ CMakeProfileDBApp::CMakeProfileDBApp(void)
                   m_WordDefaultScoreThreshold(0), m_OutDbName(kEmptyStr), m_MaxFileSize(0),
                   m_OutDbType(kEmptyStr), m_CreateIndexFile(false),m_GapOpenPenalty(0),
                   m_GapExtPenalty(0), m_PssmScaleFactor(0),m_Matrix(kEmptyStr),  m_op_mode(op_invalid),
-                  m_binary_scoremat(false), m_Taxids(new CTaxIdSet()), m_Done(false),
+                  m_binary_scoremat(false), m_DbVer(eBDB_Version5),
+                  m_Taxids(new CTaxIdSet()), m_Done(false),
                   m_ObsrvThreshold(0), m_ExcludeInvalid(false), m_CurrFreqOffset(0), m_CurrObsrOffset(0),
-                  m_UpdateFreqRatios(true)
+                  m_UpdateFreqRatios(true), m_UseModelThreshold(true)
 {
 	CRef<CVersion> version(new CVersion());
 	version->SetVersionInfo(new CBlastVersion());
@@ -378,12 +382,20 @@ void CMakeProfileDBApp::x_SetupArgDescriptions(void)
     						"Minimum word score to add a word to the lookup table",
     						CArgDescriptions::eDouble,
     						NStr::DoubleToString(kDefaultWordScoreThreshold));
+    arg_desc->AddFlag(kUseCmdlineThreshold, "Use cmdline threshold", true);
 
     arg_desc->SetCurrentGroup("Output options");
     arg_desc->AddOptionalKey(kOutDbName, "database_name",
                              "Name of database to be created\n" +
                               dflt , CArgDescriptions::eString);
 
+    arg_desc->AddDefaultKey("blastdb_version", "version",
+                             "Version of BLAST database to be created",
+                             CArgDescriptions::eInteger,
+                             NStr::NumericToString(static_cast<int>(eBDB_Version5)));
+    arg_desc->SetConstraint("blastdb_version",
+                            new CArgAllow_Integers(eBDB_Version4, eBDB_Version5));
+
     arg_desc->AddDefaultKey(kMaxFileSize, "max_file_size_in_bytes",
                             "Maximum file size for database files",
                             CArgDescriptions::eString, kDefaultMaxFileSize);
@@ -429,6 +441,18 @@ void CMakeProfileDBApp::x_SetupArgDescriptions(void)
                             "not pass validation test",
                             CArgDescriptions::eBoolean, kDefaultExcludeInvalid);
 
+    arg_desc->SetCurrentGroup("Taxonomy options");
+    arg_desc->AddOptionalKey("taxid", "TaxID",
+                             "Taxonomy ID to assign to all sequences",
+                             CArgDescriptions::eInteger);
+    arg_desc->SetConstraint("taxid", new CArgAllowValuesGreaterThanOrEqual(0));
+    arg_desc->SetDependency("taxid", CArgDescriptions::eExcludes, "taxid_map");
+
+    arg_desc->AddOptionalKey("taxid_map", "TaxIDMapFile",
+             "Text file mapping sequence IDs to taxonomy IDs.\n"
+             "Format:<SequenceId> <TaxonomyId><newline>",
+             CArgDescriptions::eInputFile);
+
     SetupArgDescriptions(arg_desc.release());
 }
 
@@ -511,6 +535,19 @@ void CMakeProfileDBApp::x_InitProgramParameters(void)
 	m_ObsrvThreshold = args[kObsrThreshold].AsDouble();
 	m_ExcludeInvalid = args[kExcludeInvalid].AsBoolean();
 
+	if (args[kUseCmdlineThreshold]){
+		m_UseModelThreshold = false;
+	}
+    m_DbVer = static_cast<EBlastDbVersion>(args["blastdb_version"].AsInteger());
+
+    if (args["taxid"].HasValue()) {
+        _ASSERT( !args["taxid_map"].HasValue() );
+        m_Taxids.Reset(new CTaxIdSet(args["taxid"].AsInteger()));
+    } else if (args["taxid_map"].HasValue()) {
+        _ASSERT( !args["taxid"].HasValue() );
+        _ASSERT( !m_Taxids.Empty() );
+        m_Taxids->SetMappingFromFile(args["taxid_map"].AsInputFile());
+    }
 }
 
 vector<string> CMakeProfileDBApp::x_GetSMPFilenames(void)
@@ -632,7 +669,7 @@ bool CMakeProfileDBApp::x_IsUpdateFreqRatios(const CPssm & p)
 void CMakeProfileDBApp::x_InitOutputDb(void)
 {
 	CWriteDB::EIndexType index_type = (m_CreateIndexFile == true ? CWriteDB::eDefault : CWriteDB::eNoIndex);
-	m_OutputDb.Reset(new CWriteDB(m_OutDbName, CWriteDB::eProtein, m_Title, index_type, m_CreateIndexFile));
+	m_OutputDb.Reset(new CWriteDB(m_OutDbName, CWriteDB::eProtein, m_Title, index_type, m_CreateIndexFile, false, false, m_DbVer));
 	m_OutputDb->SetMaxFileSize(m_MaxFileSize);
 	return;
 }
@@ -845,7 +882,7 @@ void CMakeProfileDBApp::x_RPSUpdateStatistics(CPssmWithParameters & seq, Int4 se
     	     // asn1 default value is 1
     	     m_RpsDbInfo.scale_factor = 1.0;
     	 }
-    	 if(pssm_w_parameters.GetPssm().GetFinalData().IsSetWordScoreThreshold())
+    	 if(m_UseModelThreshold && pssm_w_parameters.GetPssm().GetFinalData().IsSetWordScoreThreshold())
     	 {
     	 	 wordScoreThreshold = pssm_w_parameters.GetPssm().GetFinalData().GetWordScoreThreshold();
     	 }
@@ -1348,7 +1385,7 @@ int CMakeProfileDBApp::x_Run(void)
                 }
             }
 
-			if(pssm.GetFinalData().IsSetWordScoreThreshold()) {
+			if(m_UseModelThreshold && pssm.GetFinalData().IsSetWordScoreThreshold()) {
 			 	m_RpsDbInfo.lookup->threshold = m_RpsDbInfo.scale_factor * pssm_w_parameters.GetPssm().GetFinalData().GetWordScoreThreshold();
 			}
 			else {
