@@ -1,4 +1,4 @@
-/*  $Id: alnvec.cpp 563008 2018-05-01 11:28:43Z grichenk $
+/*  $Id: alnvec.cpp 577167 2018-12-31 20:16:49Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1075,7 +1075,12 @@ int CAlnVec::CalculatePercentIdentity(TSeqPos aln_pos) const
         }
         total += *i_res;
     }
-    return 100 * max / total;
+    if (total) {
+        return 100 * max / total;
+    }
+    else {
+        return 0;
+    }
 }
 
 
