@@ -1,4 +1,4 @@
-/*  $Id: struc_comm_field.cpp 492413 2016-02-17 15:49:17Z bollin $
+/*  $Id: struc_comm_field.cpp 590281 2019-07-26 00:10:53Z kans $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -471,8 +471,9 @@ bool CStructuredCommentField::IsValid(const CUser_object& obj, const string& des
         return false;
     }
 
-    try {
-        const CComment_rule& rule = comment_rules->FindCommentRule(prefix);
+    CConstRef<CComment_rule> ruler = comment_rules->FindCommentRuleEx(prefix);
+    if (ruler) {
+        const CComment_rule& rule = *ruler;
 
         if (rule.GetRequire_order()) {
             CComment_rule::TErrorList errors = rule.IsValid(obj);
@@ -493,10 +494,8 @@ bool CStructuredCommentField::IsValid(const CUser_object& obj, const string& des
                 return false;
             }
         }
-    } catch (CException) {
-        // no rule for this prefix
-        return false;
     }
+    return false;
 }
 
 
@@ -509,11 +508,10 @@ void CStructuredCommentField::ReorderFields(CUser_object& obj)
         return;
     }
 
-    try {
-        const CComment_rule& rule = comment_rules->FindCommentRule(prefix);
+    CConstRef<CComment_rule> ruler = comment_rules->FindCommentRuleEx(prefix);
+    if (ruler) {
+        const CComment_rule& rule = *ruler;
         rule.ReorderFields(obj);
-    } catch (CException) {
-        // no rule for this prefix
     }
 }
 
