@@ -1,4 +1,4 @@
-/*  $Id: flat_file_generator.cpp 578289 2019-01-16 16:33:46Z ivanov $
+/*  $Id: flat_file_generator.cpp 606748 2020-04-27 11:07:58Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -310,6 +310,15 @@ void CFlatFileGenerator::Generate
                 if ( m_Ctx->GetConfig().GeneRNACDSFeatures() ) {
                     flags |= CSeqEntryIndex::fGeneRNACDSOnly;
                 }
+                if ( m_Ctx->GetConfig().IsPolicyInternal() ) {
+                    policy = CSeqEntryIndex::eInternal;
+                }
+                if ( m_Ctx->GetConfig().IsPolicyExternal() ) {
+                    policy = CSeqEntryIndex::eExternal;
+                }
+                if ( m_Ctx->GetConfig().IsPolicyExhaustive() ) {
+                    policy = CSeqEntryIndex::eExhaustive;
+                }
                 CRef<CSeqEntryIndex> idx(new CSeqEntryIndex( topseh, policy, flags ));
                 m_Ctx->SetSeqEntryIndex(idx);
                 if (idx->IsIndexFailure()) {
@@ -590,6 +599,11 @@ void CFlatFileGenerator::Generate
  CFlatItemOStream& item_os,
  bool useSeqEntryIndexing)
 {
+    // useSeqEntryIndexing argument also set by relevant flags in CFlatFileConfig
+    if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
+        useSeqEntryIndexing = true;
+    }
+
     if (useSeqEntryIndexing) {
         m_Ctx->SetConfig().SetUseSeqEntryIndexer(true);
     } else {
@@ -598,7 +612,24 @@ void CFlatFileGenerator::Generate
 
     _ASSERT(entry  &&  entry.Which() != CSeq_entry::e_not_set);
 
-    if ( m_Ctx->GetConfig().BasicCleanup() )
+    const CFlatFileConfig& cfg = m_Ctx->GetConfig();
+
+    bool doNuc = false;
+    bool doProt = false;
+
+    // doNuc and doProt arguments also set by relevant flags in CFlatFileConfig
+    if ( cfg.IsViewNuc() ) {
+        doNuc = true;
+    }
+    if ( cfg.IsViewProt() ) {
+        doProt = true;
+    }
+    if ( cfg.IsViewAll() ) {
+        doNuc = true;
+        doProt = true;
+    }
+
+    if ( cfg.BasicCleanup() )
     {
 
         entry.GetTopLevelEntry().GetCompleteObject();
@@ -653,7 +684,7 @@ void CFlatFileGenerator::Generate
     // If there is a ICancel callback, wrap the item_os so
     // that every call checks it.
     const ICanceled * pCanceled = 
-        m_Ctx->GetConfig().GetCanceledCallback();
+        cfg.GetCanceledCallback();
     if( pCanceled ) {
         pItemOS.Reset( 
             new CCancelableFlatItemOStreamWrapper(
@@ -664,7 +695,7 @@ void CFlatFileGenerator::Generate
     SAnnotSelector sel = m_Ctx->SetAnnotSelector();
     m_Ctx->SetEntry(entry);
 
-    if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
+    if ( cfg.UseSeqEntryIndexer() ) {
         // CSeq_entry& top = const_cast<CSeq_entry&> (*topent);
         CSeq_entry_Handle topseh = entry.GetTopLevelEntry();
         if (m_Ctx->UsingSeqEntryIndex()) {
@@ -682,21 +713,30 @@ void CFlatFileGenerator::Generate
             try {
                 CSeqEntryIndex::EPolicy policy = CSeqEntryIndex::eAdaptive;
                 CSeqEntryIndex::TFlags flags = CSeqEntryIndex::fDefault;
-                if ( m_Ctx->GetConfig().OnlyNearFeatures() ) {
+                if ( cfg.OnlyNearFeatures() ) {
                     policy = CSeqEntryIndex::eInternal;
                 }
-                if ( m_Ctx->GetConfig().HideSNPFeatures() ) {
+                if ( cfg.HideSNPFeatures() ) {
                     flags |= CSeqEntryIndex::fHideSNPFeats;
                 }
-                if ( m_Ctx->GetConfig().HideCDDFeatures() ) {
+                if ( cfg.HideCDDFeatures() ) {
                     flags |= CSeqEntryIndex::fHideCDDFeats;
                 }
-                if ( m_Ctx->GetConfig().ShowSNPFeatures() ) {
+                if ( cfg.ShowSNPFeatures() ) {
                     flags |= CSeqEntryIndex::fShowSNPFeats;
                 }
-                if ( m_Ctx->GetConfig().ShowCDDFeatures() ) {
+                if ( cfg.ShowCDDFeatures() ) {
                     flags |= CSeqEntryIndex::fShowCDDFeats;
                 }
+                if ( m_Ctx->GetConfig().IsPolicyInternal() ) {
+                    policy = CSeqEntryIndex::eInternal;
+                }
+                if ( m_Ctx->GetConfig().IsPolicyExternal() ) {
+                    policy = CSeqEntryIndex::eExternal;
+                }
+                if ( m_Ctx->GetConfig().IsPolicyExhaustive() ) {
+                    policy = CSeqEntryIndex::eExhaustive;
+                }
                 CRef<CSeqEntryIndex> idx(new CSeqEntryIndex( topseh, policy, flags ));
                 m_Ctx->SetSeqEntryIndex(idx);
                 if (idx->IsIndexFailure()) {
@@ -710,7 +750,7 @@ void CFlatFileGenerator::Generate
         }
     }
 
-    CFlatFileConfig::TFormat format = m_Ctx->GetConfig().GetFormat();
+    CFlatFileConfig::TFormat format = cfg.GetFormat();
     CRef<CFlatItemFormatter> formatter(CFlatItemFormatter::New(format));
     if ( !formatter ) {
         NCBI_THROW(CFlatException, eInternal, "Unable to initialize formatter");
@@ -724,7 +764,7 @@ void CFlatFileGenerator::Generate
     }
 
     // this version of Gather calls method with internal Bioseq iterator
-    gatherer->Gather(*m_Ctx, *pItemOS, entry, useSeqEntryIndexing);
+    gatherer->Gather(*m_Ctx, *pItemOS, entry, useSeqEntryIndexing, doNuc, doProt);
 
     /// reset the context, but preserve our selector
     /// we do this a bit oddly since resetting the context erases the selector;
@@ -827,6 +867,33 @@ void CFlatFileGenerator::Generate
     Generate(entry, *item_os, useSeqEntryIndexing);
 }
 
+void CFlatFileGenerator::Generate
+(const CSeq_loc& loc,
+ CScope& scope,
+ CNcbiOstream& os,
+ bool useSeqEntryIndexing)
+{
+    CBioseq_Handle bsh = GetBioseqFromSeqLoc(loc, scope);
+    if (!bsh) {
+        NCBI_THROW(CFlatException, eInvalidParam, "location not in scope");
+    }
+    CSeq_entry_Handle entry = bsh.GetParentEntry();
+    if (!entry) {
+        NCBI_THROW(CFlatException, eInvalidParam, "Id not in scope");
+    }
+    CRef<CSeq_loc> location(new CSeq_loc);
+    location->Assign(loc);
+    m_Ctx->SetLocation(location);
+
+    CFlatFileConfig& cfg = m_Ctx->SetConfig();
+    if (cfg.IsStyleNormal()) {
+        cfg.SetStyleMaster();
+    }
+
+    Generate(entry, os, useSeqEntryIndexing);
+}
+
+
 
 //void CFlatFileGenerator::Reset(void)
 //{
@@ -981,5 +1048,10 @@ CFlatFileGenerator::CCancelableFlatItemOStreamWrapper::AddItem(
     m_pUnderlying->AddItem(item);
 }
 
+void CFlatFileGenerator::SetConfig(const CFlatFileConfig& cfg)
+{
+    m_Ctx->SetConfig(cfg);
+}
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
