@@ -1,4 +1,4 @@
-/*  $Id: reference_item.cpp 578290 2019-01-16 16:34:11Z ivanov $
+/*  $Id: reference_item.cpp 604101 2020-03-23 12:20:44Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -96,6 +96,30 @@ private:
 
 /////////////////////////////////////////////////////////////////////////////
 
+CCacheItem::CCacheItem(CBioseqContext& ctx, TCache csh, int length, bool is_prot) :
+    m_Cache(csh),
+    m_Length(length),
+    m_IsProt(is_prot),
+    CFlatItem(&ctx)
+{
+}
+
+IFlatItem::EItem CCacheItem::GetItemType(void) const
+{
+    return eItem_Cache;
+}
+
+
+
+void CCacheItem::Format
+(IFormatter& formatter,
+ IFlatTextOStream& text_os) const
+{
+    formatter.FormatCache(*this, text_os);
+}
+
+
+/////////////////////////////////////////////////////////////////////////////
 
 void CReferenceItem::FormatAffil(const CAffil& affil, string& result, bool gen_sub)
 {
@@ -667,13 +691,7 @@ void CReferenceItem::x_GatherInfo(CBioseqContext& ctx)
                 case CPub::e_Muid:
                     {
                         const int muid = pub.GetMuid();
-
-                        const int pmid = mlaClient.AskUidtopmid(muid);
-                        if( pmid > 0 ) {
-                            CPubMedId req(pmid);
-                            CMLAClient::TReply reply;
-                            new_pub = mlaClient.AskGetpubpmid(req, &reply);
-                        }
+                        // RW-1040: removed mlaClient.AskUidtopmid and AskGetpubpmid
                     }
                     break;
                 default:
