@@ -1,4 +1,4 @@
-/*  $Id: fasta.cpp 568147 2018-07-31 18:08:59Z gotvyans $
+/*  $Id: fasta.cpp 600608 2020-01-23 17:32:17Z foleyjp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -81,6 +81,7 @@
 #include <objtools/readers/message_listener.hpp>
 #include <objtools/readers/line_error.hpp>
 #include <objtools/readers/fasta_reader_utils.hpp>
+#include <objtools/readers/mod_reader.hpp>
 
 #include <ctype.h>
 
@@ -507,6 +508,7 @@ void CFastaReader::SetMaxIDLength(Uint4 max_len)
     CFastaDeflineReader::s_MaxAccessionLength = m_MaxIDLength = max_len;
 }
 
+
 // For reasons of efficiency, this method does not use 
 // CRef<CSeq_interval> to access range information - RW-26
 void CFastaReader::ParseDefLine(const TStr& defLine,
@@ -610,9 +612,6 @@ void CFastaReader::ParseDefLine(const TStr& s, ILineErrorListener * pMessageList
     PostProcessIDs(data.ids, s, data.has_range, data.range_start, data.range_end);
 
     m_BestID = FindBestChoice(GetIDs(), CSeq_id::BestRank);
-    if(data.has_range) {
-        m_ExpectedEnd = data.range_end - data.range_start;
-    }
     FASTA_PROGRESS("Processing Seq-id: " <<
         ( m_BestID ? m_BestID->AsFastaString() : string("UNKNOWN") ) );
 
@@ -716,13 +715,8 @@ void CFastaReader::ParseTitle(
 
     CreateWarningsForSeqDataInTitle(lineInfo.m_sLineText,lineInfo.m_iLineNum, pMessageListener);
 
-    CRef<CSeqdesc> desc(new CSeqdesc);
     CTempString title(lineInfo.m_sLineText.data(), lineInfo.m_sLineText.length());
-    NStr::TruncateSpacesInPlace(title);
-    desc->SetTitle().assign(title.data(), title.length());
-    m_CurrentSeq->SetDescr().Set().push_back(desc);
-
-    x_ApplyAllMods(*m_CurrentSeq, lineInfo.m_iLineNum, pMessageListener);
+    x_ApplyMods(title, lineInfo.m_iLineNum, *m_CurrentSeq, pMessageListener);
 }
 
 
@@ -1098,12 +1092,19 @@ void CFastaReader::x_CloseGap(
             }
         }
 
+       
+        const auto cit = m_GapsizeToLinkageEvidence.find(len);
+        const auto& gap_linkage_evidence = 
+            (cit != m_GapsizeToLinkageEvidence.end()) ?
+            cit->second :
+            m_DefaultLinkageEvidence;
+
         TGapRef pGap( new SGap(
             pos, len,
             eKnownSize,
             LineNumber(),
             m_gap_type,
-            m_gap_linkage_evidence));
+            gap_linkage_evidence));
 
         m_Gaps.push_back(pGap);
         m_TotalGapLength += len;
@@ -1232,7 +1233,9 @@ bool CFastaReader::ParseGapLine(
     if (m_gap_type && modKeyValueMultiMap.empty()) // fall back to default values coming from caller
     {
         pGapType = m_gap_type;
-        setOfLinkageEvidence = m_gap_linkage_evidence;
+        if (!m_GapsizeToLinkageEvidence.empty()) {
+            setOfLinkageEvidence = m_GapsizeToLinkageEvidence.begin()->second;
+        }
         for (const auto& rec : CSeq_gap::GetNameToGapTypeInfoMap())
         {
             if (rec.second.m_eType == *pGapType)
@@ -1269,9 +1272,11 @@ bool CFastaReader::ParseGapLine(
                     "gap-type",
                     sValue );
             }
+            continue;
 
-        } else if( canonicalKey == "linkage-evidence") {
-
+        } 
+        
+        if( canonicalKey == "linkage-evidence") {
             // could be semi-colon separated
             vector<CTempString> arrLinkageEvidences;
             NStr::Split(sValue, ";", arrLinkageEvidences, 
@@ -1295,15 +1300,15 @@ bool CFastaReader::ParseGapLine(
                         sValue );
                 }
             }
+            continue;
+        } 
 
-        } else {
-            // unknown mod.
-            FASTA_WARNING_EX(
-                LineNumber(),
-                "Unknown gap modifier name(s): " << sKey,
-                ILineError::eProblem_UnrecognizedQualifierName,
-                "gapline", sKey, kEmptyStr );
-        }
+        // unknown mod.
+        FASTA_WARNING_EX(
+            LineNumber(),
+            "Unknown gap modifier name(s): " << sKey,
+            ILineError::eProblem_UnrecognizedQualifierName,
+            "gapline", sKey, kEmptyStr );
     }
 
     if( bConflictingGapTypes ) {
@@ -1521,12 +1526,12 @@ void CFastaReader::AssembleSeq(ILineErrorListener * pMessageListener)
                             CSeq_gap::TLinkage_evidence::value_type pNewLinkEvid(
                                 new CLinkage_evidence );
                             pNewLinkEvid->SetType( *link_evid_it );
-                            vecLinkEvids.push_back(pNewLinkEvid);
+                            vecLinkEvids.push_back(move(pNewLinkEvid));
                         }
                     }
                 }
             }
-            delta_ext.Set().push_back(gap_ds);
+            delta_ext.Set().push_back(move(gap_ds));
 
             TSeqPos next_start = (i == n-1) ? m_CurrentPos : m_Gaps[i+1]->m_uPos;
             if (next_start != m_Gaps[i]->m_uPos) {
@@ -1546,9 +1551,39 @@ void CFastaReader::AssembleSeq(ILineErrorListener * pMessageListener)
     }
 }
 
+static void s_AddBiomol(CMolInfo::EBiomol biomol, CBioseq& bioseq)
+{
+    auto pDesc = Ref(new CSeqdesc());
+    pDesc->SetMolinfo().SetBiomol(biomol);
+    bioseq.SetDescr().Set().emplace_back(move(pDesc));
+}
+
+static bool sRefineNaMol(const char* beginSeqData, const char* endSeqData, CBioseq& bioseq) 
+{
+
+    auto& seqInst = bioseq.SetInst();
+    const bool hasT = 
+        (find_if(beginSeqData, endSeqData, [](char c) { return (c=='t' || c == 'T'); }) != endSeqData);
+    const bool hasU =
+        (find_if(beginSeqData, endSeqData, [](char c) { return (c=='u' || c == 'U'); }) != endSeqData);
+
+    if (hasT && !hasU) {
+        seqInst.SetMol(CSeq_inst::eMol_dna);
+        s_AddBiomol(CMolInfo::eBiomol_genomic, bioseq); // RW-931
+        return true;
+    }
+
+    if (hasU && !hasT) {
+        seqInst.SetMol(CSeq_inst::eMol_rna);
+        return true;
+    }
+
+    return false;
+}
+
+
 void CFastaReader::AssignMolType(ILineErrorListener * pMessageListener)
 {
-    CSeq_inst&                  inst = m_CurrentSeq->SetInst();
     CSeq_inst::EMol             default_mol;
     CFormatGuess::ESTStrictness strictness;
 
@@ -1566,23 +1601,42 @@ void CFastaReader::AssignMolType(ILineErrorListener * pMessageListener)
     default:            strictness = CFormatGuess::eST_Default; break;
     }
 
+    auto& inst = m_CurrentSeq->SetInst();
+
     if (TestFlag(fForceType)) {
         _ASSERT(default_mol != CSeq_inst::eMol_not_set);
         inst.SetMol(default_mol);
         return;
-    } else if (inst.IsSetMol()) {
-        return; // previously found an informative ID
-    } else if (m_SeqData.empty()) {
+    } 
+        
+    if (inst.IsSetMol()) {
+        if (inst.GetMol() == CSeq_inst::eMol_na && !m_SeqData.empty()) {
+            sRefineNaMol(m_SeqData.data(), 
+                    m_SeqData.data() + min(m_SeqData.length(), SIZE_TYPE(4096)), 
+                    *m_CurrentSeq);
+        }
+        return;
+    } 
+        
+    if (m_SeqData.empty()) {
         // Nothing else to go on, but that's OK (no sequence to worry
         // about encoding); however, Seq-inst.mol is still mandatory.
         inst.SetMol(CSeq_inst::eMol_not_set);
         return;
     }
 
+
     // Do the residue frequencies suggest a specific type?
     SIZE_TYPE length = min(m_SeqData.length(), SIZE_TYPE(4096));
-    switch (CFormatGuess::SequenceType(m_SeqData.data(), length, strictness)) {
-    case CFormatGuess::eNucleotide:  inst.SetMol(CSeq_inst::eMol_na);  return;
+    const auto& data = m_SeqData.data();
+    switch (CFormatGuess::SequenceType(data, length, strictness)) {
+    case CFormatGuess::eNucleotide:  
+    {
+        if (!sRefineNaMol(data, data+length, *m_CurrentSeq)) {
+            inst.SetMol(CSeq_inst::eMol_na);
+        }
+        return;
+    }
     case CFormatGuess::eProtein:     inst.SetMol(CSeq_inst::eMol_aa);  return;
     default:
         if (default_mol == CSeq_inst::eMol_not_set) {
@@ -1896,6 +1950,26 @@ CRef<CSeq_entry> ReadFasta(CNcbiIstream& in, TReadFastaFlags flags,
     return seq_entry;
 }
 
+CRef<CSeq_entry> ReadFasta(CNcbiIstream& in, TReadFastaFlags flags,
+                           int* counter, CFastaReader::TMasks* lcv,
+                           ILineErrorListener* pMessageListener)
+{
+    CFastaReader reader(in, flags);
+    if (counter) {
+        reader.SetIDGenerator().SetCounter(*counter);
+    }
+    if (lcv) {
+        reader.SaveMasks(lcv);
+    }
+
+    auto pEntry = reader.ReadSet(kMax_Int, pMessageListener);
+
+    if (counter) {
+        *counter = reader.GetIDGenerator().GetCounter();
+    }
+    return pEntry;
+}
+
 
 IFastaEntryScan::~IFastaEntryScan()
 {
@@ -1962,8 +2036,7 @@ void CFastaMapper::AssembleSeq(ILineErrorListener * pMessageListener)
 void ReadFastaFileMap(SFastaFileMap* fasta_map, CNcbiIfstream& input)
 {
     static const CFastaReader::TFlags kFlags
-        = CFastaReader::fAssumeNuc | CFastaReader::fAllSeqIds
-        | CFastaReader::fNoSeqData;
+        = CFastaReader::fAssumeNuc | CFastaReader::fNoSeqData;
 
     if ( !input.is_open() ) {
         return;
@@ -2005,9 +2078,9 @@ CSourceModParser* CFastaReader::xCreateSourceModeParser(
     ILineErrorListener* pErrorListener)
 {
     // first of all, honor any explicit fBadModThrow flag given to the reader:
-    if (TestFlag(fBadModThrow) || TestFlag(fUnknModThrow)) {
-        return new CSourceModParser(CSourceModParser::eHandleBadMod_Throw);
-    }
+//    if (TestFlag(fBadModThrow) || TestFlag(fUnknModThrow)) {
+//        return new CSourceModParser(CSourceModParser::eHandleBadMod_Throw);
+//    }
     // otherwise, try to construct the parser around any given error listener:
     if (pErrorListener != nullptr) {
         return new CSourceModParser(pErrorListener, m_uLineNumber+1);
@@ -2016,80 +2089,82 @@ CSourceModParser* CFastaReader::xCreateSourceModeParser(
     return new CSourceModParser(CSourceModParser::eHandleBadMod_Ignore);
 }
 
-void CFastaReader::x_ApplyAllMods( 
-    CBioseq & bioseq,
-    TSeqPos iLineNum,
-    ILineErrorListener * pMessageListener )
+
+static void s_AppendMods(
+        const CModHandler::TModList& mods, 
+        string& title
+        ) 
 {
-    // this is called even if there the user did not request
-    // mods to be added because we want to give a warning if there
-    // are mods when not expected.
+    for (const auto& mod : mods) {
 
-    auto_ptr<CSourceModParser> pSmp(xCreateSourceModeParser(pMessageListener));
-    pSmp->SetModFilter( m_pModFilter );
-    CRef<CSeqdesc> title_desc;
-
-    if( ! bioseq.IsSetDescr() && ! bioseq.GetDescr().IsSet() ) {
-        return;
-    }
-
-    // find title
-    // (and remember iter in case we need to delete later)
-    CSeq_descr::Tdata & desc_container = bioseq.SetDescr().Set();
-    CSeq_descr::Tdata::iterator desc_it = desc_container.begin();
-    const CSeq_descr::Tdata::iterator desc_end = desc_container.end();
-    for( ; desc_it != desc_end; ++desc_it ) {
-        if( (*desc_it)->IsTitle() ) {
-            title_desc.Reset( &(**desc_it) );
-            break;
-        }
-    }
-
-    if ( ! title_desc ) {
-        return;
-    }
-
-    string& title = title_desc->SetTitle();
-
-    if( TestFlag(fAddMods) ) {
-        title = pSmp->ParseTitle(title, CConstRef<CSeq_id>(bioseq.GetFirstId()) );
-
-        pSmp->ApplyAllMods(bioseq);
-        ApplySourceMods(bioseq, *pSmp);
-        pSmp->GetLabel(&title, CSourceModParser::fUnusedMods);
-
-        copy( pSmp->GetBadMods().begin(), pSmp->GetBadMods().end(),
-            inserter(m_BadMods, m_BadMods.begin()) );
-        CSourceModParser::TMods unused_mods = 
-            pSmp->GetMods(CSourceModParser::fUnusedMods);
-        copy( unused_mods.begin(), unused_mods.end(),
-            inserter(m_UnusedMods, m_UnusedMods.begin() ) );
-    } else {
-        // user did not request fAddMods, so we warn if we found
-        // mods anyway
-        pSmp->ParseTitle(
-            title, 
-            CConstRef<CSeq_id>(bioseq.GetFirstId()),
-            1 // "1" since we only care whether or not there are mods, not how many
-            );
-        CSourceModParser::TMods unused_mods = pSmp->GetMods(CSourceModParser::fUnusedMods);
-        if( ! unused_mods.empty() ) {
-            FASTA_WARNING(iLineNum,
-                "FASTA-Reader: Ignoring FASTA modifier(s) found because "
-                "the input was not expected to have any.",
-                ILineError::eProblem_ModifierFoundButNoneExpected,
-                "defline");
-        }
-    }
-
-    NStr::TruncateSpacesInPlace(title);
-
-    // remove title if empty
-    if( title.empty() ) {
-        desc_container.erase(desc_it);
+        title.append(" [" 
+                + mod.GetName()
+                + "=" 
+                + mod.GetValue()
+                + "]");
     }
 }
 
+
+void CFastaReader::SetExcludedMods(const vector<string>& excluded_mods)
+{
+    m_ModHandler.SetExcludedMods(excluded_mods);
+}
+
+
+void CFastaReader::x_ApplyMods(     
+     const string& title, 
+     TSeqPos line_number, 
+     CBioseq& bioseq,
+     ILineErrorListener* pMessageListener )
+{
+    string processed_title = title;
+    if (TestFlag(fAddMods)) {
+        string remainder;
+        CModHandler::TModList mods;
+        CTitleParser::Apply(processed_title, mods, remainder);
+
+        const auto pFirstID = bioseq.GetFirstId();
+        _ASSERT(pFirstID != nullptr);
+        const auto idString = pFirstID->AsFastaString();
+
+        CDefaultModErrorReporter 
+            errorReporter(idString, line_number, pMessageListener);
+
+        CModHandler::TModList rejected_mods;
+        m_ModHandler.Clear();
+        m_ModHandler.AddMods(mods, CModHandler::eReplace, rejected_mods, errorReporter);
+        s_AppendMods(rejected_mods, remainder);
+
+        CModHandler::TModList skipped_mods;
+        const bool logInfo = 
+            pMessageListener ?
+            pMessageListener->SevEnabled(eDiag_Info) :
+            false;
+        CModAdder::Apply(m_ModHandler, bioseq, skipped_mods, logInfo, errorReporter);
+        s_AppendMods(skipped_mods, remainder);
+
+        processed_title = remainder;
+    }
+    else
+    if (!TestFlag(fIgnoreMods) &&
+        CTitleParser::HasMods(title)) {
+        FASTA_WARNING(line_number,
+        "FASTA-Reader: Ignoring FASTA modifier(s) found because "
+        "the input was not expected to have any.",
+        ILineError::eProblem_ModifierFoundButNoneExpected,
+        "defline");
+    }
+
+    NStr::TruncateSpacesInPlace(processed_title);
+    if (!processed_title.empty()) {
+        auto pDesc = Ref(new CSeqdesc());
+        pDesc->SetTitle() = processed_title;
+        bioseq.SetDescr().Set().push_back(move(pDesc));
+    }
+}
+
+
 std::string CFastaReader::x_NucOrProt(void) const
 {
     if( m_CurrentSeq && m_CurrentSeq->IsSetInst() && 
@@ -2121,7 +2196,7 @@ string CFastaReader::CanonicalizeString(const TStr & sValue)
     return newString;
 }
 
-// static
+  
 CFastaReader::SGap::SGap(
     TSeqPos uPos,
     TSignedSeqPos uLen,
@@ -2138,18 +2213,38 @@ CFastaReader::SGap::SGap(
 {
 }
 
+
+void CFastaReader::SetGapLinkageEvidence(
+    CSeq_gap::EType type,
+    const set<int>& defaultEvidence,
+    const map<TSeqPos, set<int>>& countToEvidenceMap 
+)
+{
+    SetGapLinkageEvidences(type, defaultEvidence);
+
+    m_GapsizeToLinkageEvidence.clear();
+    for (const auto& key_val : countToEvidenceMap) {
+        const auto& input_evidence_set = key_val.second;
+        auto& evidence_set = m_GapsizeToLinkageEvidence[key_val.first];
+        for (const auto& evidence : input_evidence_set) {
+            evidence_set.insert(static_cast<CLinkage_evidence::EType>(evidence));
+        }
+    }
+}
+
+
 void CFastaReader::SetGapLinkageEvidences(CSeq_gap::EType type, const set<int>& evidences)
 {
-   if (type == -1)
-      m_gap_type.Release();
-   else
-      m_gap_type.Reset(new SGap::TGapTypeObj(type));
+    if (type == -1)
+        m_gap_type.Release();
+    else 
+         m_gap_type.Reset(new SGap::TGapTypeObj(type));
 
-   m_gap_linkage_evidence.clear();
-   ITERATE(set<int>, it, evidences)
-   {
-       m_gap_linkage_evidence.insert((CLinkage_evidence::EType)*it);
-   }
+
+    m_DefaultLinkageEvidence.clear();
+    for (const auto& evidence : evidences) {
+        m_DefaultLinkageEvidence.insert(static_cast<CLinkage_evidence::EType>(evidence));
+    }
 }
 
 
@@ -2183,10 +2278,6 @@ void CFastaReader::IgnoreProblem(ILineError::EProblem problem)
     m_ignorable.push_back(problem);
 }
 
-void CFastaReader::ApplySourceMods(CBioseq&, CSourceModParser&)
-{
-}
-
 
 END_SCOPE(objects)
 END_NCBI_SCOPE
