@@ -1,4 +1,4 @@
-/*  $Id: fasta_reader_utils.cpp 566929 2018-07-10 16:29:24Z foleyjp $
+/*  $Id: fasta_reader_utils.cpp 599582 2020-01-02 20:02:39Z foleyjp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -39,6 +39,7 @@
 #include <objects/general/Dbtag.hpp>
 #include <objects/general/Object_id.hpp>
 #include <objtools/readers/fasta_reader_utils.hpp>
+#include <objtools/readers/seqid_validate.hpp>
 
 #define NCBI_USE_ERRCODE_X Objtools_Rd_Fasta // Will need to change this 
 
@@ -49,23 +50,6 @@ size_t CFastaDeflineReader::s_MaxLocalIDLength = CSeq_id::kMaxLocalIDLength;
 size_t CFastaDeflineReader::s_MaxGeneralTagLength = CSeq_id::kMaxGeneralTagLength;
 size_t CFastaDeflineReader::s_MaxAccessionLength = CSeq_id::kMaxAccessionLength;
 
-static bool s_IsValidLocalID(const CSeq_id& id, const CSeqIdCheck::TInfo& info) 
-{
-    if (id.IsLocal()) {
-        if (id.GetLocal().IsId()) {
-            return true;
-        }
-        if (id.GetLocal().IsStr()) {
-            auto id_string = id.GetLocal().GetStr();
-            auto string_to_check = (info.fFastaFlags & CFastaReader::fQuickIDCheck) ?
-                                    id_string.substr(0,1) :
-                                    id_string;
-            return !(CSeq_id::CheckLocalID(string_to_check)&CSeq_id::fInvalidChar);
-        }
-    }
-
-    return false;
-}
 
 
 static void s_PostError(ILineErrorListener* pMessageListener,
@@ -143,17 +127,49 @@ void CFastaDeflineReader::ParseDefline(const CTempString& defline,
     ParseDefline(defline, info, data, pMessageListener, fn_idcheck);
 }
 
+/*
+class CDeprecatedIdCheckWrapper {
+public:
+    using FDeprecatedIdCheck = CFastaDeflineReader::FIdCheck;
+    using TParseInfo = CFastaDeflineReader::SDeflineParseInfo;
+
+    CDeprecatedIdCheckWrapper(
+            FDeprecatedIdCheck fDeprecatedCheck, 
+            const TParseInfo& parseInfo, 
+            ILineErrorListener* pMessageListener) : 
+        m_fDeprecatedCheck(fDeprecatedCheck), 
+        m_ParseInfo(parseInfo),
+        m_pMessageListener(pMessageListener) {}
+
+    using TIds = list<CRef<CSeq_id>>;
+    using FReportError = CFastaIdValidate::FReportError;
+
+
+    void operator()(const TIds& ids, 
+            int lineNum, 
+            FReportError //fReportError
+        ) {
+        m_ParseInfo.lineNumber = lineNum;
+        m_fDeprecatedCheck(ids, m_ParseInfo, m_pMessageListener);
+    }
+
+private:
+    TParseInfo m_ParseInfo;
+    FDeprecatedIdCheck m_fDeprecatedCheck=nullptr;
+    ILineErrorListener* m_pMessageListener=nullptr;
+};
+*/
 
 void CFastaDeflineReader::ParseDefline(const CTempString& defline,
         const SDeflineParseInfo& info,
         SDeflineData& data,
         ILineErrorListener* pMessageListener,
         FIdCheck fn_idcheck) 
-    {
-        size_t range_len = 0;
-        const TFastaFlags& fFastaFlags = info.fFastaFlags;
-        const TSeqPos& lineNumber = info.lineNumber;
-        data.has_range = false;
+{
+    size_t range_len = 0;
+    const TFastaFlags& fFastaFlags = info.fFastaFlags;
+    const TSeqPos& lineNumber = info.lineNumber;
+    data.has_range = false;
 
         const size_t len = defline.length();
         if (len <= 1 || 
@@ -263,6 +279,7 @@ void CFastaDeflineReader::ParseDefline(const CTempString& defline,
             data.ids, 
             pMessageListener,
             fn_idcheck);
+
         data.has_range = (range_len>0);
     }
 
@@ -333,13 +350,88 @@ TSeqPos CFastaDeflineReader::ParseRange(
     return TSeqPos(s.length() - pos);
 }
 
+static bool s_ASCII_IsUnAmbigNuc(unsigned char c)
+{
+    switch( c ) {
+    case 'A':
+    case 'C':
+    case 'G':
+    case 'T':
+    case 'a':
+    case 'c':
+    case 'g':
+    case 't':
+        return true;
+    default:
+        return false;
+    }
+}
+
+
+class CIdErrorReporter
+{
+public:
+    CIdErrorReporter(ILineErrorListener* pMessageListener, bool ignoreGeneralParsingError=false);
+
+    void operator() (EDiagSev severity, 
+            int lineNum, 
+            const string& idString,
+            CFastaIdValidate::EErrCode errCode,
+            const string& msg);
+private:
+    using TCodePair = pair<CObjReaderLineException::EProblem, CObjReaderParseException::EErrCode>;
+    ILineErrorListener* m_pMessageListener = nullptr;
+    bool m_IgnoreGeneralParsingError=false;
+};
+
+
+
+CIdErrorReporter::CIdErrorReporter(ILineErrorListener* pMessageListener, bool ignoreGeneralParsingError) : 
+    m_pMessageListener(pMessageListener), m_IgnoreGeneralParsingError(ignoreGeneralParsingError) {}
+
+
+void CIdErrorReporter::operator()(EDiagSev severity,
+        int lineNum,
+        const string& idString,
+        CFastaIdValidate::EErrCode errCode,
+        const string& msg)
+{
+
+    static map<CFastaIdValidate::EErrCode,TCodePair> s_CodeMap = /* replace with compile-time map */
+    {
+     {CFastaIdValidate::eIDTooLong,{ILineError::eProblem_GeneralParsingError, CObjReaderParseException::eIDTooLong}},
+     {CFastaIdValidate::eBadLocalID,{ILineError::eProblem_GeneralParsingError, CObjReaderParseException::eInvalidID}},
+     {CFastaIdValidate::eUnexpectedNucResidues,{ILineError::eProblem_UnexpectedNucResidues, CObjReaderParseException::eFormat}},
+     {CFastaIdValidate::eUnexpectedAminoAcids,{ILineError::eProblem_UnexpectedAminoAcids, CObjReaderParseException::eFormat}}
+    };
+
+
+    const auto cit = s_CodeMap.find(errCode);
+    _ASSERT(cit != s_CodeMap.end()); // convert this to a compile-time assertion 
+
+    const auto& problem = cit->second.first;
+    if (m_IgnoreGeneralParsingError &&
+        problem == ILineError::eProblem_GeneralParsingError) {
+        return;
+    }
+
+    const auto& parseExceptionCode = cit->second.second;
+    if (severity == eDiag_Error) {
+        s_PostError(m_pMessageListener, lineNum, msg, problem, parseExceptionCode);
+    }
+    else {
+        s_PostWarning(m_pMessageListener, lineNum, msg, problem, parseExceptionCode);
+    }
+}
+
 
 void CFastaDeflineReader::x_ProcessIDs(
     const CTempString& id_string,
     const SDeflineParseInfo& info,
     TIds& ids,
     ILineErrorListener* pMessageListener,
-    FIdCheck f_id_check)
+    FIdCheck f_id_check
+    )
 {
     if (info.fBaseFlags & CReaderBase::fAllIdsAsLocal) 
     {
@@ -424,7 +516,7 @@ bool CFastaDeflineReader::ParseIDs(
         flags |= CSeq_id::fParse_RawText;
     }
 
-    const bool ignoreError
+    const bool ignoreGeneralParsingError
         = (find(ignoredErrors.cbegin(), ignoredErrors.cend(), ILineError::eProblem_GeneralParsingError) 
            != ignoredErrors.cend()); 
 
@@ -442,7 +534,7 @@ bool CFastaDeflineReader::ParseIDs(
                 + ", the sequence contains 'comma' symbol and replaced with 'underscore' "
                 + "symbol. Please find and correct the sequence id.";
 
-            if (!ignoreError) {
+            if (!ignoreGeneralParsingError) {
                 s_PostWarning(pMessageListener, 
                             info.lineNumber,
                             errMessage,
@@ -459,41 +551,16 @@ bool CFastaDeflineReader::ParseIDs(
         // swap(ids, old_ids);
     }
 
-
-    
     if ( info.fBaseFlags & CReaderBase::fNumericIdsAsLocal ) {
         x_ConvertNumericToLocal(ids);
     }
 
-    if (num_ids == 1) {
-        if (ids.front()->IsLocal()) 
-        {
-            if (!s_IsValidLocalID(*ids.front(), info)) {
-                ids.clear();
-                return false;
-            }
-        }
-        if ( x_ExceedsMaxLength(s, info.maxIdLength) ) {
-            const string errMessage =
-                "Near line " + NStr::NumericToString(info.lineNumber)
-                + ", the sequence ID is too long.  Its length is " + NStr::NumericToString(s.length())
-                + " but the max length allowed is "+  NStr::NumericToString(info.maxIdLength)
-                + ".  Please find and correct all sequence IDs that are too long.";
 
-
-            if (!ignoreError) {
-                s_PostError(pMessageListener, 
-                    info.lineNumber,
-                    errMessage,
-                    ILineError::eProblem_GeneralParsingError, 
-                    CObjReaderParseException::eIDTooLong);
-            }
-
-            if (x_ExcessiveSeqDataInTitle(s, info.fFastaFlags)) {
-                return false;
-            }
-        }
-    }
+    CFastaIdValidate idValidate(info.fFastaFlags);
+    idValidate.SetMaxLocalIDLength(info.maxIdLength);
+    idValidate.SetMaxGeneralTagLength(info.maxIdLength);
+    idValidate.SetMaxAccessionLength(info.maxIdLength);
+    idValidate(ids, info.lineNumber, CIdErrorReporter(pMessageListener, ignoreGeneralParsingError));
 
     return true;
 }
@@ -511,35 +578,6 @@ void CFastaDeflineReader::x_ConvertNumericToLocal(
 }
 
 
-bool 
-CFastaDeflineReader::x_ExceedsMaxLength(const CTempString& title,
-    const TSeqPos max_length)
-{
-    auto last = title.rfind('|');
-    auto substring = (last == NPOS) ? title : title.substr(last+1);
-
-    return (substring.length() > max_length);
-}
-
-
-static bool s_ASCII_IsUnAmbigNuc(unsigned char c)
-{
-    switch( c ) {
-    case 'A':
-    case 'C':
-    case 'G':
-    case 'T':
-    case 'a':
-    case 'c':
-    case 'g':
-    case 't':
-        return true;
-    default:
-        return false;
-    }
-}
-
-
 void CSeqIdCheck::operator()(const TIds& ids, 
                              const TInfo& info,
                              ILineErrorListener* listener) 
@@ -548,210 +586,8 @@ void CSeqIdCheck::operator()(const TIds& ids,
         return;
     }
 
-    x_CheckForExcessiveSeqData(*(ids.back()), info, listener);
-
-    for (const auto& pId : ids) {
-        if (pId->IsLocal() && 
-            !x_IsValidLocalID(*pId, info)) {
-            NCBI_THROW2(CObjReaderParseException, eInvalidID, 
-                "'" + pId->GetSeqIdString() + "' is not a valid local ID", 0);
-        }
-        x_CheckIDLength(*pId, info, listener);
-    }
-}
-
-
-bool CSeqIdCheck::x_IsValidLocalID(const CSeq_id& id, 
-                                 const TInfo& info)
-{
-    return s_IsValidLocalID(id, info);
-}
-
-
-void CSeqIdCheck::x_PostIDLengthError(const size_t id_length,
-                                      const string& type_string,
-                                      const size_t max_length,
-                                      const TInfo& info,
-                                      ILineErrorListener* listener)
-{
-
-    const string& err_message =
-        "Near line " + NStr::NumericToString(info.lineNumber) +
-        + ", the " + type_string + " is too long.  Its length is " + NStr::NumericToString(id_length)
-        + " but the maximum allowed " + type_string + " length is "+  NStr::NumericToString(max_length)
-        + ".  Please find and correct all " + type_string + "s that are too long.";
-
-    s_PostError(listener, 
-        info.lineNumber,
-        err_message,
-        ILineError::eProblem_GeneralParsingError,
-        CObjReaderParseException::eIDTooLong);
-
-}
-
-
-void CSeqIdCheck::x_CheckIDLength(const CSeq_id& id, 
-                                    const TInfo& info,
-                                    ILineErrorListener* listener) 
-{
-    if (id.IsLocal()) {
-        if (id.GetLocal().IsStr() &&
-            id.GetLocal().GetStr().length() > CFastaDeflineReader::s_MaxLocalIDLength) {
-            x_PostIDLengthError(id.GetLocal().GetStr().length(),
-                                "local id", CFastaDeflineReader::s_MaxLocalIDLength,
-                                info, listener);
-        }
-        return;
-    }
-
-
-    if (id.IsGeneral()) {
-        if (id.GetGeneral().IsSetTag() &&
-            id.GetGeneral().GetTag().IsStr()) {
-            const auto length = id.GetGeneral().GetTag().GetStr().length();
-            if (length > CFastaDeflineReader::s_MaxGeneralTagLength) {
-                x_PostIDLengthError(length,
-                                    "general id string", CFastaDeflineReader::s_MaxGeneralTagLength,
-                                    info, listener);
-            }
-        }
-        return;
-    }
-
-
-   auto pTextId = id.GetTextseq_Id();
-   if (pTextId &&
-       pTextId->IsSetAccession()) {
-        const auto length = pTextId->GetAccession().length();
-        if (length > CFastaDeflineReader::s_MaxAccessionLength) {
-            x_PostIDLengthError(length,
-                                "accession", CFastaDeflineReader::s_MaxAccessionLength, 
-                                info, listener);
-        }
-   }
-}
-
-
-void CSeqIdCheck::x_CheckForExcessiveSeqData(const CSeq_id& id,
-                                               const TInfo& info,
-                                               ILineErrorListener* listener)
-{
-    const auto& id_string = id.GetSeqIdString();
-
-    const TSeqPos kWarnNumNucCharsAtEnd = 20;
-    const TSeqPos kWarnNumAminoAcidCharsAtEnd = 50;
-    const TSeqPos kErrNumNucCharsAtEnd = 25;
-
-    const bool assume_prot = (info.fFastaFlags & CFastaReader::fAssumeProt);
-
-    if (!assume_prot && id_string.length() > kWarnNumNucCharsAtEnd) {
-        TSeqPos numNucChars = 0;
-        for (size_t i = id_string.size(); i>0; i--) {
-            const auto ch = id_string[i - 1];
-            if (!s_ASCII_IsUnAmbigNuc(ch) && (ch != 'N')) {
-                break;
-            }
-            ++numNucChars;
-        }
-
-        if (numNucChars > kWarnNumNucCharsAtEnd) {
-            const string err_message = 
-            "Fasta Reader: sequence id ends with " +
-            NStr::NumericToString(numNucChars) +
-            " valid nucleotide characters. " +
-            " Was the sequence accidentally placed in the definition line?";    
-       
-            if (numNucChars > kErrNumNucCharsAtEnd) {
-                s_PostError(listener,
-                            info.lineNumber,
-                            err_message,
-                            ILineError::eProblem_UnexpectedNucResidues,
-                            CObjReaderParseException::eFormat);
-            
-            } else{ 
-                s_PostWarning(listener,
-                              info.lineNumber,
-                              err_message,
-                              ILineError::eProblem_UnexpectedNucResidues,
-                              CObjReaderParseException::eFormat);
-            }
-            return;        
-        }
-    }
-
-    const bool assume_nuc = (info.fFastaFlags & CFastaReader::fAssumeNuc);
-    // Check for Aa sequence
-    if (!assume_nuc && id_string.length() > kWarnNumAminoAcidCharsAtEnd) {
-        TSeqPos numAaChars = 0;
-        for (size_t i = id_string.size(); i>0; i--) {
-            const auto ch = id_string[i - 1];
-            if ( !(ch >= 'A' && ch <= 'Z')  &&
-                 !(ch >= 'a' && ch <= 'z') ) {
-                break;
-            }
-            ++numAaChars;
-        }
-        if (numAaChars > kWarnNumAminoAcidCharsAtEnd) {
-            const string err_message = 
-            "Fasta Reader: sequence id ends with " +
-            NStr::NumericToString(numAaChars) +
-            " valid amino-acid characters. " +
-            " Was the sequence accidentally placed in the definition line?";    
-        
-            s_PostWarning(listener,
-                          info.lineNumber,
-                          err_message,
-                          ILineError::eProblem_UnexpectedAminoAcids,
-                          CObjReaderParseException::eFormat);
-        }
-    }
-}
-
-
-
-bool 
-CFastaDeflineReader::x_ExcessiveSeqDataInTitle(const CTempString& title, 
-    TFastaFlags fasta_flags) 
-{
-    if (fasta_flags & CFastaReader::fAssumeProt) {
-        return false;
-    }
-
-    // Check for nuc or aa sequence at the end of the title
-    const TSeqPos kWarnNumNucCharsAtEnd = 20;
-    const TSeqPos kWarnNumAminoAcidCharsAtEnd = 50;
-
-    // Check for nuc sequence
-    if (title.length() > kWarnNumNucCharsAtEnd) {
-        TSeqPos numNucChars = 0;
-        for (size_t i = title.size(); i>0; i--) {
-            const auto ch = title[i-1];
-            if (!s_ASCII_IsUnAmbigNuc(ch) && (ch != 'N')) {
-                break;
-            }
-            ++numNucChars;
-        }
-        if (numNucChars > kWarnNumNucCharsAtEnd) {
-            return true;
-        }
-    }
-
-    // Check for Aa sequence
-    if (title.length() > kWarnNumAminoAcidCharsAtEnd) {
-        TSeqPos numAaChars = 0;
-        for (size_t i=title.size(); i>0; i--) {
-            const auto ch = title[i-1];
-            if ( !(ch >= 'A' && ch <= 'Z')  &&
-                 !(ch >= 'a' && ch <= 'z') ) {
-                break;
-            }
-            ++numAaChars;
-        }
-        if (numAaChars > kWarnNumAminoAcidCharsAtEnd) {
-            return true;
-        }
-    }
-    return false;
+    CFastaIdValidate s_IdValidate(info.fFastaFlags);
+    s_IdValidate(ids, info.lineNumber, CIdErrorReporter(listener));
 }
 
 
@@ -761,7 +597,6 @@ CRef<CSeq_id> CFastaIdHandler::GenerateID(bool unique_id)
 }
 
 
-
 CRef<CSeq_id> CFastaIdHandler::GenerateID(const string& defline, const bool unique_id)
 {
     const bool advance = true;
@@ -809,7 +644,6 @@ CRef<CSeq_id> CSeqIdGenerator::GenerateID(void) const
 }
 
 
-
 END_SCOPE(objects)
 END_NCBI_SCOPE
 
