@@ -1,4 +1,4 @@
-/*  $Id: gff2_data.cpp 575507 2018-11-29 19:36:24Z ivanov $
+/*  $Id: gff2_data.cpp 607807 2020-05-07 18:58:43Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -61,25 +61,11 @@
 #include <objtools/readers/gff2_reader.hpp>
 #include <objtools/readers/gff3_reader.hpp>
 
+#include <objtools/readers/reader_message.hpp>
 BEGIN_NCBI_SCOPE
 
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
-//  ----------------------------------------------------------------------------
-CCdregion::EFrame FramePlusToMinus(
-    CCdregion::EFrame frame)
-//  ----------------------------------------------------------------------------
-{
-    static map<CCdregion::EFrame, CCdregion::EFrame> mFramePlusToMinus;
-    if (mFramePlusToMinus.empty()) {
-        mFramePlusToMinus[CCdregion::eFrame_not_set] = CCdregion::eFrame_not_set;
-        mFramePlusToMinus[CCdregion::eFrame_one] = CCdregion::eFrame_three;
-        mFramePlusToMinus[CCdregion::eFrame_two] = CCdregion::eFrame_two;
-        mFramePlusToMinus[CCdregion::eFrame_three] = CCdregion::eFrame_one;
-    }
-    return mFramePlusToMinus[frame];
-}
-
 //  ----------------------------------------------------------------------------
 CRef<CCode_break> s_StringToCodeBreak(
     const string& str,
@@ -174,43 +160,11 @@ CBioSource::EGenome s_StringToGenome(
     return CBioSource::eGenome_unknown;
 }
     
-//  ----------------------------------------------------------------------------
-CGff2Record::CGff2Record():
-    m_uSeqStart( 0 ),
-    m_uSeqStop( 0 ),
-    m_pdScore( 0 ),
-    m_peStrand( 0 ),
-    m_pePhase( 0 )
-//  ----------------------------------------------------------------------------
-{
-};
-
-//  ----------------------------------------------------------------------------
-CGff2Record::~CGff2Record()
-//  ----------------------------------------------------------------------------
-{
-    delete m_pdScore;
-    delete m_peStrand;
-    delete m_pePhase; 
-};
-
-
-unsigned int CGff2Record::m_nextId(0);
 //  -----------------------------------------------------------------------------
-unsigned int CGff2Record::NextId()
+void CGff2Record::TokenizeGFF(
+    vector<CTempStringEx>& columns, 
+    const CTempStringEx& in_line)
 //  -----------------------------------------------------------------------------
-{
-    return ++m_nextId;
-}
-
-//  -----------------------------------------------------------------------------
-void CGff2Record::ResetId()
-//  -----------------------------------------------------------------------------
-{
-    m_nextId = 0;
-}
-
-void CGff2Record::TokenizeGFF(vector<CTempStringEx>& columns, const CTempStringEx& in_line)
 {
     columns.clear();
     columns.reserve(9);
@@ -274,7 +228,7 @@ bool CGff2Record::AssignFromGff(
     }
     //  to do: more sanity checks
 
-    columns[0].Copy(m_strId, 0, CTempString::npos);
+    columns[0].Copy(mSeqId, 0, CTempString::npos);
     columns[1].Copy(m_strSource, 0, CTempString::npos);
     columns[2].Copy(m_strType, 0, CTempString::npos);
 
@@ -283,23 +237,21 @@ bool CGff2Record::AssignFromGff(
     m_uSeqStop = NStr::StringToUInt( columns[4] ) - 1;
     }
     catch (const CException&) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
+        string message = "Bad data line: Both \"start\" and \"stop\" must be positive integers.";
+        CReaderMessage error(
             eDiag_Error,
             0,
-            "Bad data line: Both \"start\" and \"stop\" must be positive integers.",
-            ILineError::eProblem_FeatureBadStartAndOrStop));
-        pErr->Throw();
+            message);
+        throw error;
     }
     if (m_uSeqStop < m_uSeqStart) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
+        string message = "Bad data line: location start is greater than location stop (start="
+            + string(columns[3]) + ", stop=" + string(columns[4]) + ").";
+        CReaderMessage error(
             eDiag_Error,
             0,
-            "Bad data line: location start is greater than location stop (start="
-            + string(columns[3]) + ", stop=" + string(columns[4]) + ").",
-            ILineError::eProblem_FeatureBadStartAndOrStop) );
-        pErr->Throw();
+            message);
+        throw error;
     }
 
     if ( columns[5] != "." ) {
@@ -376,34 +328,6 @@ bool CGff2Record::GetAttribute(
     return !values.empty();
 }
 
-//  ----------------------------------------------------------------------------
-CRef<CSeq_id> CGff2Record::GetSeqId(
-    int flags,
-    SeqIdResolver seqidresolve ) const
-//  ----------------------------------------------------------------------------
-{
-    if (!seqidresolve) {
-        seqidresolve = CReadUtil::AsSeqId;
-    }
-    return seqidresolve(Id(), flags, true);
-}
-
-//  ----------------------------------------------------------------------------
-CRef<CSeq_loc> CGff2Record::GetSeqLoc(
-    int flags,
-    SeqIdResolver seqidresolve ) const
-//  ----------------------------------------------------------------------------
-{
-    CRef<CSeq_loc> pLocation(new CSeq_loc);
-    pLocation->SetInt().SetId(*GetSeqId(flags, seqidresolve));
-    pLocation->SetInt().SetFrom(static_cast<TSeqPos>(SeqStart()));
-    pLocation->SetInt().SetTo(static_cast<TSeqPos>(SeqStop()));
-    if (IsSetStrand()) {
-        pLocation->SetInt().SetStrand(Strand());
-    }
-    return pLocation;
-}
-
 //  ----------------------------------------------------------------------------
 string CGff2Record::xNormalizedAttributeKey(
     const CTempString& strRawKey )
@@ -582,17 +506,12 @@ bool CGff2Record::InitializeFeature(
     int flags,
     CRef<CSeq_feat> pFeature,
     SeqIdResolver seqidresolve ) const
-//  ----------------------------------------------------------------------------
+    //  ----------------------------------------------------------------------------
 {
-    return (
-        xInitFeatureLocation(flags, pFeature, seqidresolve)  &&
-        xInitFeatureData(flags, pFeature)  &&
-        xMigrateId(pFeature)  &&
-        xMigrateStartStopStrand(pFeature)  &&
-        xMigrateType(pFeature)  &&
-        xMigrateScore(pFeature)  &&
-        xMigratePhase(pFeature)  &&
-        xMigrateAttributes(flags, pFeature) );
+    if (!CGffBaseColumns::InitializeFeature(flags, pFeature, seqidresolve)) {
+        return false;
+    }
+    return xMigrateAttributes(flags, pFeature);
 }
 
 //  ----------------------------------------------------------------------------
@@ -684,52 +603,6 @@ bool CGff2Record::xUpdateFeatureData(
     return true;
 }
     
-
-//  ----------------------------------------------------------------------------
-bool CGff2Record::xMigrateId(
-    CRef<CSeq_feat> pFeature ) const
-//  ----------------------------------------------------------------------------
-{
-    unsigned int featId = NextId();
-    CRef<CFeat_id> pFeatId(new CFeat_id);
-    pFeatId->SetLocal().SetId(featId);
-    pFeature->SetId(*pFeatId);
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGff2Record::xMigrateStartStopStrand(
-    CRef<CSeq_feat> pFeature ) const
-//  ----------------------------------------------------------------------------
-{
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGff2Record::xMigrateType(
-    CRef<CSeq_feat> pFeature ) const
-//  ----------------------------------------------------------------------------
-{
-    return true;
-}
-
-
-//  ----------------------------------------------------------------------------
-bool CGff2Record::xMigrateScore(
-    CRef<CSeq_feat> pFeature ) const
-//  ----------------------------------------------------------------------------
-{
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGff2Record::xMigratePhase(
-    CRef<CSeq_feat> pFeature ) const
-//  ----------------------------------------------------------------------------
-{
-    return true;
-}
-
 //  ----------------------------------------------------------------------------
 bool CGff2Record::xMigrateAttributes(
     int flags,
@@ -958,6 +831,45 @@ bool CGff2Record::xMigrateAttributes(
         attrs_left.erase(it);
     }
 
+    it = attrs_left.find("regulatory_class");
+    if (it != attrs_left.end()) {
+        if (pFeature->GetData().IsImp()  &&  (pFeature->GetData().GetImp().GetKey()  == "regulatory")) {
+            //pFeature->SetData().SetImp().SetKey(it->second);
+            pFeature->RemoveQualifier("regulatory_class");
+            pFeature->AddQualifier("regulatory_class", it->second);
+            attrs_left.erase(it);
+        }
+        //pFeature->RemoveQualifier("regulatory_class");
+    }
+
+    it = attrs_left.find("rpt_type");
+    if (it != attrs_left.end()) {
+        map<string, string> satellites = {
+            {"microsatellite", "microsatellite"},
+            {"minisatellite", "minisatellite"},
+            {"satellite_DNA", "satellite"},
+            {"satellite", "satellite"},
+        };
+        pFeature->RemoveQualifier("rpt_type");
+        pFeature->RemoveQualifier("satellite");
+        auto rpt_type = it->second;
+        auto satelliteIt = satellites.find(rpt_type);
+        if (satelliteIt != satellites.end()) {
+            pFeature->AddQualifier("satellite", satelliteIt->second);
+        }
+        else {
+            pFeature->AddQualifier("rpt_type", rpt_type);
+        }
+        attrs_left.erase(it);
+    }
+
+    it = attrs_left.find("satellite");
+    if (it != attrs_left.end()) {
+        if (pFeature->GetData().IsImp()  &&  pFeature->GetData().GetImp().GetKey() == "repeat_region") {
+            attrs_left.erase(it);
+        }           
+    }
+
     it = attrs_left.find("transl_except");
     if (it != attrs_left.end()) {
         if (pFeature->GetData().IsCdregion()) {
@@ -988,6 +900,10 @@ bool CGff2Record::xMigrateAttributes(
         }
     }
 
+    if (!xMigrateAttributesGo(flags, pFeature, attrs_left)) {
+		return false;
+	}
+
     if (pFeature->GetData().IsBiosrc()) { 
         if (!xMigrateAttributesSubSource(flags, pFeature, attrs_left)) {
             return false;
@@ -1051,6 +967,9 @@ bool CGff2Record::xMigrateAttributeDefault(
     NStr::Split(it->second, ",", values, 0);
     for (list<CTempStringEx>::const_iterator cit = values.begin(); cit != values.end();
             cit++) {
+        if (cit->empty()) {
+            continue;
+        }
         string value = xNormalizedAttributeValue(*cit);
         pFeature->AddQualifier(qualKey, value);
     }
@@ -1125,6 +1044,29 @@ bool CGff2Record::xMigrateAttributesOrgName(
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CGff2Record::xMigrateAttributesGo(
+    int flags,
+    CRef<CSeq_feat> pFeature,
+    TAttributes& attrs) const
+//  ----------------------------------------------------------------------------
+{
+    for (auto it = attrs.begin(); it != attrs.end(); /**/) {
+        if (NStr::StartsWith(it->first, "go_")) {
+            try {
+                CReadUtil::AddGeneOntologyTerm(*pFeature, it->first, it->second);
+            }
+            catch(ILineError&) {
+            }
+            it = attrs.erase(it);
+        }
+        else {
+            it++;
+        }
+    }
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 bool CGff2Record::xMigrateAttributesSubSource(
     int flags,
@@ -1199,72 +1141,42 @@ bool CGff2Record::xMigrateAttributesSubSource(
     return true;
 }
 
-//  ----------------------------------------------------------------------------
-bool CGff2Record::xInitFeatureLocation(
-    int flags,
-    CRef<CSeq_feat> pFeature,
-    SeqIdResolver seqidresolve ) const
-//  ----------------------------------------------------------------------------
-{
-    pFeature->SetLocation(*GetSeqLoc(flags, seqidresolve));
-    return true;
-}
-
 //  ----------------------------------------------------------------------------
 bool CGff2Record::xInitFeatureData(
     int flags,
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
-    bool invalidFeaturesToRegion = !(flags & CGff2Reader::fGenbankMode);
-
-    string gbkey;
-    if (GetAttribute("gbkey", gbkey)) {
-        if (gbkey == "Src") {
-            pFeature->SetData().SetBiosrc();
+    if (Type() == "region") {
+        string gbkey;
+        if (GetAttribute("gbkey", gbkey)) {
+            if (gbkey == "Src") {
+                pFeature->SetData().SetBiosrc();
+                return true;
+            }
+            // regardless of gbkey (rw-1062)
+            pFeature->SetData().SetRegion();
             return true;
         }
     }
 
-    if (Type() == "ncRNA") {
-        string qual_type;
-        if(GetAttribute("ncrna_class", qual_type)) {
-            if (qual_type == "other") {
-                qual_type = "ncRNA";
-            }
-            if (CSoMap::SoTypeToFeature(
-                    qual_type, *pFeature, invalidFeaturesToRegion)) {
-                return true;
-            }
-        }
-    }
     auto recognizedType = Type();
     if (recognizedType == "start_codon"  || recognizedType == "stop_codon") {
         recognizedType = "cds";
     }
-    if (!CSoMap::SoTypeToFeature(
-            recognizedType, *pFeature, invalidFeaturesToRegion)) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-                eDiag_Error,
-                0,
-                string("Bad data line: Invalid feature type \"") + recognizedType + "\"",
-                ILineError::eProblem_UnrecognizedFeatureName) );
-        pErr->Throw();
-    }
 
-    CSeqFeatData::ESubtype subtype = pFeature->GetData().GetSubtype();
-    if (subtype == CSeqFeatData::eSubtype_cdregion) {
-        CCdregion::EFrame frame = Phase();
-        if (frame == CCdregion::eFrame_not_set) {
-            frame = CCdregion::eFrame_one;
-        }
-        pFeature->SetData().SetCdregion().SetFrame(frame);
-        return true;
+    bool invalidFeaturesToRegion = !(flags & CGff2Reader::fGenbankMode);
+    if (!CSoMap::SoTypeToFeature(
+        recognizedType, *pFeature, invalidFeaturesToRegion)) {
+        string message = "Bad data line: Invalid feature type \"" + recognizedType + "\"";
+        CReaderMessage error(
+            eDiag_Error,
+            0,
+            message);
+        throw error;
     }
-    return true;
+    return CGffBaseColumns::xInitFeatureData(flags, pFeature);
 }
 
 END_objects_SCOPE
-
 END_NCBI_SCOPE
