@@ -1,4 +1,4 @@
-/*  $Id: gff3_reader.cpp 575509 2018-11-29 19:37:34Z ivanov $
+/*  $Id: gff3_reader.cpp 607807 2020-05-07 18:58:43Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -26,30 +26,18 @@
  * Author:  Frank Ludwig
  *
  * File Description:
- *   GFF file reader
+ *   GFF3 file reader
  *
  */
 
 #include <ncbi_pch.hpp>
 #include <corelib/ncbistd.hpp>
-#include <corelib/ncbiapp.hpp>
-#include <corelib/ncbithr.hpp>
-#include <corelib/ncbiutil.hpp>
-#include <corelib/ncbiexpt.hpp>
-#include <corelib/stream_utils.hpp>
 
-#include <util/static_map.hpp>
 #include <util/line_reader.hpp>
 
-#include <serial/iterator.hpp>
-#include <serial/objistrasn.hpp>
 
-// Objects includes
-#include <objects/general/Int_fuzz.hpp>
 #include <objects/general/Object_id.hpp>
 #include <objects/general/User_object.hpp>
-#include <objects/general/User_field.hpp>
-#include <objects/general/Dbtag.hpp>
 
 #include <objects/seqloc/Seq_id.hpp>
 #include <objects/seqloc/Seq_loc.hpp>
@@ -58,48 +46,23 @@
 
 #include <objects/seq/Seq_annot.hpp>
 #include <objects/seq/Annot_id.hpp>
-#include <objects/seq/Annotdesc.hpp>
 #include <objects/seq/Annot_descr.hpp>
-#include <objects/seq/Seq_descr.hpp>
 #include <objects/seq/so_map.hpp>
-#include <objects/seqfeat/SeqFeatData.hpp>
 #include <objects/seqfeat/SeqFeatXref.hpp>
 
 #include <objects/seqfeat/Seq_feat.hpp>
-#include <objects/seqfeat/BioSource.hpp>
-#include <objects/seqfeat/Org_ref.hpp>
-#include <objects/seqfeat/OrgName.hpp>
-#include <objects/seqfeat/SubSource.hpp>
-#include <objects/seqfeat/OrgMod.hpp>
-#include <objects/seqfeat/Gene_ref.hpp>
 #include <objects/seqfeat/Code_break.hpp>
 #include <objects/seqfeat/Genetic_code.hpp>
-#include <objects/seqfeat/Genetic_code_table.hpp>
 #include <objects/seqfeat/RNA_ref.hpp>
-#include <objects/seqfeat/Trna_ext.hpp>
-#include <objects/seqfeat/Imp_feat.hpp>
-#include <objects/seqfeat/Gb_qual.hpp>
 #include <objects/seqfeat/Feat_id.hpp>
-#include <objects/seqset/Bioseq_set.hpp>
 
-#include <objtools/readers/read_util.hpp>
-#include <objtools/readers/reader_exception.hpp>
-#include <objtools/readers/line_error.hpp>
-#include <objtools/readers/message_listener.hpp>
-#include <objtools/readers/gff2_reader.hpp>
 #include <objtools/readers/gff3_reader.hpp>
-#include <objtools/readers/gff2_data.hpp>
-#include <objtools/error_codes.hpp>
+#include "reader_message_handler.hpp"
 
 #include <algorithm>
 
-//#include "gff3_data.hpp"
-
-#define NCBI_USE_ERRCODE_X   Objtools_Rd_RepMask
-
 BEGIN_NCBI_SCOPE
-
-BEGIN_objects_SCOPE // namespace ncbi::objects::
+BEGIN_objects_SCOPE
 
 unsigned int CGff3Reader::msGenericIdCounter = 0;
 
@@ -199,19 +162,68 @@ CGff3Reader::CGff3Reader(
     unsigned int uFlags,
     const string& name,
     const string& title,
-    SeqIdResolver resolver ):
+    SeqIdResolver resolver,
+    CReaderListener* pRL):
 //  ----------------------------------------------------------------------------
-    CGff2Reader( uFlags, name, title, resolver )
+    CGff2Reader( uFlags, name, title, resolver, pRL )
 {
     CGff2Record::ResetId();
 }
 
+//  ----------------------------------------------------------------------------
+CGff3Reader::CGff3Reader(
+    unsigned int uFlags,
+    CReaderListener* pRL):
+//  ----------------------------------------------------------------------------
+    CGff3Reader(uFlags, "", "", CReadUtil::AsSeqId, pRL)
+{
+}
+
 //  ----------------------------------------------------------------------------
 CGff3Reader::~CGff3Reader()
 //  ----------------------------------------------------------------------------
 {
 }
 
+//  ----------------------------------------------------------------------------                
+CRef<CSeq_annot>
+CGff3Reader::ReadSeqAnnot(
+    ILineReader& lr,
+    ILineErrorListener* pEC ) 
+//  ----------------------------------------------------------------------------                
+{
+ 
+    mCurrentFeatureCount = 0;
+    mParsingAlignment = false;
+    auto pAnnot = CReaderBase::ReadSeqAnnot(lr, pEC);
+    if (pAnnot  &&  pAnnot->GetData().Which() == CSeq_annot::TData::e_not_set) {
+        return CRef<CSeq_annot>();
+    }
+    return pAnnot;
+}
+
+//  ----------------------------------------------------------------------------
+void
+CGff3Reader::xProcessData(
+    const TReaderData& readerData,
+    CSeq_annot& annot) 
+//  ----------------------------------------------------------------------------
+{
+    for (const auto& lineData: readerData) {
+        const auto& line = lineData.mData;
+        if (xParseStructuredComment(line)  &&  
+                !NStr::StartsWith(line, "##sequence-region") ) {
+            continue;
+        }
+        if (xParseBrowserLine(line, annot)) {
+            continue;
+        }
+        if (xParseFeature(line, annot, nullptr)) {
+            continue;
+        }
+    }
+}
+
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::IsInGenbankMode() const
 //  ----------------------------------------------------------------------------
@@ -219,10 +231,155 @@ bool CGff3Reader::IsInGenbankMode() const
     return (m_iFlags & CGff3Reader::fGenbankMode);
 }
 
+//  ----------------------------------------------------------------------------
+void
+CGff3Reader::xGetData(
+    ILineReader& lr,
+    TReaderData& readerData)
+//  ----------------------------------------------------------------------------
+{
+    readerData.clear();
+    string line;
+    if (xGetLine(lr, line)) {
+        if (xNeedsNewSeqAnnot(line)) {
+            lr.UngetLine();
+            return;
+        }
+        if (xIsTrackLine(line)) {
+            if (!mCurrentFeatureCount) {
+                xParseTrackLine(line);
+                xGetData(lr, readerData);
+                return;
+            }
+            m_PendingLine = line;
+            return;
+        }
+        if (xIsTrackTerminator(line)) {
+            if (!mCurrentFeatureCount) {
+                xParseTrackLine("track");
+                xGetData(lr, readerData);
+            }
+            return;
+        }
+        if (!xIsCurrentDataType(line)) {
+            xUngetLine(lr);
+            return;
+        }
+    readerData.push_back(TReaderLine{m_uLineNumber, line});
+    }
+    ++m_uDataCount;
+}
+
+//  ----------------------------------------------------------------------------
+void CGff3Reader::xProcessAlignmentData(
+    CSeq_annot& annot) 
+//  ----------------------------------------------------------------------------
+{
+    for (const auto id : mAlignmentData.mIds) {
+        CRef<CSeq_align> pAlign = Ref(new CSeq_align());
+        if (x_MergeAlignments(mAlignmentData.mAlignments.at(id), pAlign)) {
+            // if available, add current browser information
+            if ( m_CurrentBrowserInfo ) {
+                annot.SetDesc().Set().push_back( m_CurrentBrowserInfo );
+            }
+
+            annot.SetNameDesc("alignments");
+
+            if ( !m_AnnotTitle.empty() ) {
+                annot.SetTitleDesc(m_AnnotTitle);
+            }
+            // Add alignment
+            annot.SetData().SetAlign().push_back(pAlign);
+        }
+    }
+}
+
+//  ----------------------------------------------------------------------------
+bool
+CGff3Reader::xParseFeature(
+    const string& line,
+    CSeq_annot& annot,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    if (CGff2Reader::IsAlignmentData(line)) {
+        return xParseAlignment(line);
+    }
+
+    //parse record:
+    shared_ptr<CGff2Record> pRecord(x_CreateRecord());
+    try {
+        if (!pRecord->AssignFromGff(line)) {
+			return false;
+		}
+    }
+    catch(CObjReaderLineException& err) {
+        ProcessError(err, pEC);
+        return false;
+    }
+
+    //make sure we are interested:
+    if (xIsIgnoredFeatureType(pRecord->Type())) {
+        return true;
+    }
+    if (xIsIgnoredFeatureId(pRecord->Id())) {
+        return true;
+    }
+
+    //append feature to annot:
+    if (!xUpdateAnnotFeature(*pRecord, annot, pEC)) {
+        return false;
+    }
+
+    ++mCurrentFeatureCount;
+    mParsingAlignment = false;
+    return true;
+}
+
+
+//  ----------------------------------------------------------------------------
+bool CGff3Reader::xParseAlignment(
+    const string& strLine)
+//  ----------------------------------------------------------------------------
+{
+    if (IsInGenbankMode()) {
+        return true;
+    }
+    auto& ids = mAlignmentData.mIds;
+    auto& alignments = mAlignmentData.mAlignments;
+
+    unique_ptr<CGff2Record> pRecord(x_CreateRecord());
+
+    if ( !pRecord->AssignFromGff(strLine) ) {
+        return false;
+    }
+    
+    string id;
+    if ( !pRecord->GetAttribute("ID", id) ) {
+        id = pRecord->Id();
+    }
+
+    if (alignments.find(id) == alignments.end()) {
+       ids.push_back(id);
+    }
+
+    CRef<CSeq_align> alignment;
+    if (!x_CreateAlignment(*pRecord, alignment)) {
+        return false;
+    }
+
+    alignments[id].push_back(alignment);
+
+    ++mCurrentFeatureCount;
+    mParsingAlignment = true;
+    return true;
+}
+
+
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xUpdateAnnotFeature(
     const CGff2Record& record,
-    CRef< CSeq_annot > pAnnot,
+    CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
@@ -231,49 +388,61 @@ bool CGff3Reader::xUpdateAnnotFeature(
     string type = record.Type();
     NStr::ToLower(type);
     if (type == "exon" || type == "five_prime_utr" || type == "three_prime_utr") {
-        return xUpdateAnnotExon(record, pFeature, pAnnot, pEC);
+        return xUpdateAnnotExon(record, pFeature, annot, pEC);
     }
     if (type == "cds"  ||  type == "start_codon"  || type == "stop_codon") {
-        return xUpdateAnnotCds(record, pFeature, pAnnot, pEC);
+        return xUpdateAnnotCds(record, pFeature, annot, pEC);
     }
     if (type == "gene") {
-        return xUpdateAnnotGene(record, pFeature, pAnnot, pEC);
+        return xUpdateAnnotGene(record, pFeature, annot, pEC);
     }
     if (type == "mrna") {
-        return xUpdateAnnotMrna(record, pFeature, pAnnot, pEC);
+        return xUpdateAnnotMrna(record, pFeature, annot, pEC);
     }
-    return xUpdateAnnotGeneric(record, pFeature, pAnnot, pEC);
+    if (type == "region") {
+        return xUpdateAnnotRegion(record, pFeature, annot, pEC);
+    }
+    return xUpdateAnnotGeneric(record, pFeature, annot, pEC);
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff3Reader::xVerifyExonLocation(
+void CGff3Reader::xVerifyExonLocation(
     const string& mrnaId,
-    const CGff2Record& exon,
-    ILineErrorListener* pEC)
+    const CGff2Record& exon)
 //  ----------------------------------------------------------------------------
 {
-    //return true;
     map<string,CRef<CSeq_interval> >::const_iterator cit = mMrnaLocs.find(mrnaId);
     if (cit == mMrnaLocs.end()) {
-        return false;
+        string message = "Bad data line: ";
+        message += exon.Type();
+        message += " referring to non-existent parent feature.";
+        CReaderMessage error(
+            eDiag_Error,
+            m_uLineNumber,
+            message);
+        throw error;
     }
     const CSeq_interval& containingInt = cit->second.GetObject();
     const CRef<CSeq_loc> pContainedLoc = exon.GetSeqLoc(m_iFlags, mSeqIdResolve);
     const CSeq_interval& containedInt = pContainedLoc->GetInt();
-    if (containedInt.GetFrom() < containingInt.GetFrom()) {
-        return false;
+    if (containedInt.GetFrom() < containingInt.GetFrom()  ||  
+            containedInt.GetTo() > containingInt.GetTo()) {
+        string message = "Bad data line: ";
+        message += exon.Type();
+        message += " extends beyond parent feature.";
+        CReaderMessage error(
+            eDiag_Error,
+            m_uLineNumber,
+            message);
+        throw error;
     }
-    if (containedInt.GetTo() > containingInt.GetTo()) {
-        return false;
-    }
-    return true;
 }
 
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xUpdateAnnotExon(
     const CGff2Record& record,
     CRef<CSeq_feat> pFeature,
-    CRef<CSeq_annot> pAnnot,
+    CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
@@ -287,21 +456,14 @@ bool CGff3Reader::xUpdateAnnotExon(
                 xAddPendingExon(parentId, record);
                 return true;
             }
-            if (pParent->GetData().IsRna()  &&  !xVerifyExonLocation(parentId, record, pEC)) {
-                AutoPtr<CObjReaderLineException> pErr(
-                    CObjReaderLineException::Create(
-                        eDiag_Error,
-                        0,
-                        "Bad data line: Exon record referring to non-existing mRNA or gene parent.",
-                        ILineError::eProblem_FeatureBadStartAndOrStop));
-                ProcessError(*pErr, pEC);
-                return false;
+            if (pParent->GetData().IsRna()) {
+                xVerifyExonLocation(parentId, record);
             }
             if (pParent->GetData().IsGene()) {
                 if  (!xInitializeFeature(record, pFeature)) {
                     return false;
                 }
-                return xAddFeatureToAnnot(pFeature, pAnnot);            
+                return xAddFeatureToAnnot(pFeature, annot);            
             }
             IdToFeatureMap::iterator fit = m_MapIdToFeature.find(parentId);
             if (fit != m_MapIdToFeature.end()) {
@@ -320,20 +482,11 @@ bool CGff3Reader::xUpdateAnnotExon(
 bool CGff3Reader::xUpdateAnnotCds(
     const CGff2Record& record,
     CRef<CSeq_feat> pFeature,
-    CRef<CSeq_annot> pAnnot,
+    CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-    if (!xVerifyCdsParents(record)) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Bad data line: CDS record with bad parent assignments.",
-            ILineError::eProblem_GeneralParsingError) );
-        ProcessError(*pErr, pEC);
-        return false;
-    }
+    xVerifyCdsParents(record);
 
     list<string> parents;
     record.GetAttribute("Parent", parents);
@@ -342,14 +495,11 @@ bool CGff3Reader::xUpdateAnnotCds(
     // Preliminary:
     //  We do not support multiparented CDS features in -genbank mode yet.
     if (IsInGenbankMode()  &&  parents.size() > 1){
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
+        CReaderMessage error(
             eDiag_Error,
-            0,
-            "Unsupported: CDS record with multiple parents.",
-            ILineError::eProblem_GeneralParsingError) );
-        ProcessError(*pErr, pEC);
-        return false;
+            m_uLineNumber,
+            "Unsupported: CDS record with multiple parents.");
+        throw error;
     }
 
     // Step 1:
@@ -430,7 +580,7 @@ bool CGff3Reader::xUpdateAnnotCds(
                     xFeatureSetXrefGrandParent(parentId, pFeature);
                 }
             }
-            xAddFeatureToAnnot(pFeature, pAnnot);
+            xAddFeatureToAnnot(pFeature, annot);
             m_MapIdToFeature[cdsId] = pFeature;
         }
     }
@@ -520,21 +670,20 @@ bool CGff3Reader::xFindFeatureUnderConstruction(
         return false;
     }
 
-    AutoPtr<CObjReaderLineException> pErr(CObjReaderLineException::Create(
+    CReaderMessage fatal(
         eDiag_Fatal,
-        0,
-        string("Bad data line: Duplicate feature ID \"") + id + "\"",
-        ILineError::eProblem_DuplicateIDs) );
+        m_uLineNumber,
+        "Bad data line:  Duplicate feature ID \"" + id + "\".");
     if (record.Id() != mIdToSeqIdMap[id]) {
-        pErr->Throw();
+        throw fatal;
     }
     if (it->second->GetData().IsRna()) {
-        pErr->Throw();
+        throw fatal;
     }
     CSeq_feat tempFeat;
     if (CSoMap::SoTypeToFeature(record.Type(), tempFeat)) {
         if (it->second->GetData().GetSubtype() != tempFeat.GetData().GetSubtype()) {
-            pErr->Throw();
+            throw fatal;
         }
     }
 
@@ -546,7 +695,7 @@ bool CGff3Reader::xFindFeatureUnderConstruction(
 bool CGff3Reader::xUpdateAnnotGeneric(
     const CGff2Record& record,
     CRef<CSeq_feat> pFeature,
-    CRef<CSeq_annot> pAnnot,
+    CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
@@ -559,25 +708,19 @@ bool CGff3Reader::xUpdateAnnotGeneric(
     if (featType == "stop_codon_read_through"  ||  featType == "selenocysteine") {
         string cdsParent;
         if (!record.GetAttribute("Parent", cdsParent)) {
-            AutoPtr<CObjReaderLineException> pErr(
-                CObjReaderLineException::Create(
+            CReaderMessage error(
                 eDiag_Error,
-                0,
-                "Bad data line: Unassigned code break.",
-                ILineError::eProblem_GeneralParsingError) );
-            ProcessError(*pErr, pEC);
-            return false;
+                m_uLineNumber,
+                "Bad data line: Unassigned code break.");
+            throw error;
         }
         IdToFeatureMap::iterator it = m_MapIdToFeature.find(cdsParent);
         if (it == m_MapIdToFeature.end()) {
-            AutoPtr<CObjReaderLineException> pErr(
-                CObjReaderLineException::Create(
+            CReaderMessage error(
                 eDiag_Error,
-                0,
-                "Bad data line: Code break assigned to missing feature.",
-                ILineError::eProblem_GeneralParsingError) );
-            ProcessError(*pErr, pEC);
-            return false;
+                m_uLineNumber,
+                "Bad data line: Code break assigned to missing feature.");
+            throw error;
         }
 
         CRef<CCode_break> pCodeBreak(new CCode_break); 
@@ -600,14 +743,15 @@ bool CGff3Reader::xUpdateAnnotGeneric(
     if (!xInitializeFeature(record, pFeature)) {
         return false;
     }
-    if (! xAddFeatureToAnnot(pFeature, pAnnot)) {
+    if (! xAddFeatureToAnnot(pFeature, annot)) {
         return false;
     }
     string strId;
     if ( record.GetAttribute("ID", strId)) {
         m_MapIdToFeature[strId] = pFeature;
     }
-    if (pFeature->GetData().IsRna()  ||  pFeature->GetData().GetSubtype() == CSeqFeatData::eSubtype_misc_RNA) {
+    if (pFeature->GetData().IsRna()  ||  
+            pFeature->GetData().GetSubtype() == CSeqFeatData::eSubtype_misc_RNA) {
         CRef<CSeq_interval> rnaLoc(new CSeq_interval);
         rnaLoc->Assign(pFeature->GetLocation().GetInt());
         mMrnaLocs[strId] = rnaLoc;
@@ -619,7 +763,7 @@ bool CGff3Reader::xUpdateAnnotGeneric(
 bool CGff3Reader::xUpdateAnnotMrna(
     const CGff2Record& record,
     CRef<CSeq_feat> pFeature,
-    CRef<CSeq_annot> pAnnot,
+    CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
@@ -639,30 +783,27 @@ bool CGff3Reader::xUpdateAnnotMrna(
                 cit != parents.end();
                 ++cit) {
             if (!xFeatureSetXrefParent(*cit, pFeature)) {
-                AutoPtr<CObjReaderLineException> pErr(
-                    CObjReaderLineException::Create(
-                    eDiag_Warning,
-                    0,
-                    "Bad data line: mRNA record with bad parent assignment.",
-                    ILineError::eProblem_MissingContext) );
-                ProcessError(*pErr, pEC);
+                CReaderMessage error(
+                    eDiag_Error,
+                    m_uLineNumber,
+                    "Bad data line: mRNA record with bad parent assignment.");
+                throw error;
             }
         }
     }
 
-    string strId;
+    string strId;   
     if ( record.GetAttribute("ID", strId)) {
         m_MapIdToFeature[strId] = pFeature;
     }
     CRef<CSeq_interval> mrnaLoc(new CSeq_interval);
     CSeq_loc::E_Choice choice = pFeature->GetLocation().Which();
     if (choice != CSeq_loc::e_Int) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-                eDiag_Error,
-                0,
-                "Internal error: Unexpected location type.",
-                ILineError::eProblem_BadFeatureInterval));
+        CReaderMessage error(
+            eDiag_Error,
+            m_uLineNumber,
+            "Internal error: Unexpected location type.");
+        throw error;
     }
     mrnaLoc->Assign(pFeature->GetLocation().GetInt());
     mMrnaLocs[strId] = mrnaLoc;
@@ -671,9 +812,9 @@ bool CGff3Reader::xUpdateAnnotMrna(
     xGetPendingExons(strId, pendingExons);
     for (auto exonRecord: pendingExons) {
         CRef< CSeq_feat > pFeature(new CSeq_feat);
-        xUpdateAnnotExon(exonRecord, pFeature, pAnnot, pEC);
+        xUpdateAnnotExon(exonRecord, pFeature, annot, pEC);
     }
-    if (! xAddFeatureToAnnot(pFeature, pAnnot)) {
+    if (! xAddFeatureToAnnot(pFeature, annot)) {
         return false;
     }
     return true;
@@ -683,40 +824,85 @@ bool CGff3Reader::xUpdateAnnotMrna(
 bool CGff3Reader::xUpdateAnnotGene(
     const CGff2Record& record,
     CRef<CSeq_feat> pFeature,
-    CRef<CSeq_annot> pAnnot,
+    CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-    return xUpdateAnnotGeneric(record, pFeature, pAnnot, pEC);
+    CRef<CSeq_feat> pUnderConstruction(new CSeq_feat);
+    if (xFindFeatureUnderConstruction(record, pUnderConstruction)) {
+        return record.UpdateFeature(m_iFlags, pUnderConstruction);
+    }
+
+    if (!xInitializeFeature(record, pFeature)) {
+        return false;
+    }
+    if (! xAddFeatureToAnnot(pFeature, annot)) {
+        return false;
+    }
+    string strId;
+    if ( record.GetAttribute("ID", strId)) {
+        m_MapIdToFeature[strId] = pFeature;
+    }
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+bool CGff3Reader::xUpdateAnnotRegion(
+    const CGff2Record& record,
+    CRef<CSeq_feat> pFeature,
+    CSeq_annot& annot,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    if (!record.InitializeFeature(m_iFlags, pFeature)) {
+        return false;
+    }
+
+    if (! xAddFeatureToAnnot(pFeature, annot)) {
+        return false;
+    }
+    string strId;
+    if ( record.GetAttribute("ID", strId)) {
+        mIdToSeqIdMap[strId] = record.Id();
+        m_MapIdToFeature[strId] = pFeature;
+    }
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xAddFeatureToAnnot(
     CRef< CSeq_feat > pFeature,
-    CRef< CSeq_annot > pAnnot )
+    CSeq_annot& annot )
 //  ----------------------------------------------------------------------------
 {
-    pAnnot->SetData().SetFtable().push_back( pFeature ) ;
+    annot.SetData().SetFtable().push_back( pFeature ) ;
     return true;
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff3Reader::xVerifyCdsParents(
+void CGff3Reader::xVerifyCdsParents(
     const CGff2Record& record)
 //  ----------------------------------------------------------------------------
 {
     string id;
     string parents;
     if (!record.GetAttribute("ID", id)) {
-        return true;
+        return;
     }
     record.GetAttribute("Parent", parents);
     map<string, string>::iterator it = mCdsParentMap.find(id);
     if (it == mCdsParentMap.end()) {
         mCdsParentMap[id] = parents;
-        return true;
+        return;
     }
-    return (it->second == parents);
+    if (it->second == parents) {
+        return;
+    }
+    CReaderMessage error(
+        eDiag_Error,
+        m_uLineNumber,
+        "Bad data line: CDS record with bad parent assignments.");
+    throw error;
 }
 
 //  ----------------------------------------------------------------------------
@@ -873,19 +1059,48 @@ CGff3Reader::xGetPendingExons(
 
 //  ----------------------------------------------------------------------------
 void CGff3Reader::xPostProcessAnnot(
-    CRef<CSeq_annot>& pAnnot,
-    ILineErrorListener *pEC)
-    //  ----------------------------------------------------------------------------
+    CSeq_annot& annot)
+//  ----------------------------------------------------------------------------
 {
-    for (const auto& it: mPendingExons) {
-        AutoPtr<CObjReaderLineException> pErr(CObjReaderLineException::Create(
-            eDiag_Warning,
-            0,
-            "Bad data line: Record references non-existant Parent=" + it.first,
-            ILineError::eProblem_MissingContext) );
-        ProcessError(*pErr, pEC);
+    if (mAlignmentData) {
+        xProcessAlignmentData(annot);
+        return;
     }
-    return CGff2Reader::xPostProcessAnnot(pAnnot, pEC);
+    if (!mCurrentFeatureCount) {
+        return;
+    }
+
+    for (const auto& it: mPendingExons) {
+        CReaderMessage warning(
+            eDiag_Warning,
+            m_uLineNumber,
+            "Bad data line: Record references non-existant Parent=" + it.first);
+        m_pMessageHandler->Report(warning);
+    }
+    return CGff2Reader::xPostProcessAnnot(annot);
+}
+
+//  ---------------------------------------------------------------------------
+bool
+CGff3Reader::xNeedsNewSeqAnnot(
+    const string& line)
+//  ---------------------------------------------------------------------------
+{
+    if (IsInGenbankMode()) {
+        vector<string> columns;
+        NStr::Split(line, "\t ", columns, NStr::eMergeDelims);
+        string seqId = columns[0];
+        if (m_CurrentSeqId == seqId) {
+            return false;
+        }
+        m_CurrentSeqId = seqId;
+        if (mCurrentFeatureCount == 0) {
+            return false;
+        }
+        m_PendingLine = line;
+        return true;
+    }
+    return false;
 }
 
 
