@@ -1,4 +1,4 @@
-/*  $Id: idmapper_builtin.cpp 407582 2013-07-23 11:15:22Z kornbluh $
+/*  $Id: idmapper_builtin.cpp 472138 2015-07-07 16:07:55Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,7 @@ extern const char* sc_BuiltinConfig;
 
 CIdMapperBuiltin::CIdMapperBuiltin(const std::string& strContext,
                                    bool bInvert,
-                                   IMessageListener* pErrors)
+                                   ILineErrorListener* pErrors)
     : CIdMapperConfig(strContext, bInvert, pErrors)
 {
     Initialize();
